package com.loit.common.aspect;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.jms.Destination;
import javax.servlet.http.HttpServletRequest;

import org.apache.activemq.command.ActiveMQQueue;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import com.alibaba.fastjson.JSON;
import com.loit.common.constant.AuditConstants;
import com.loit.common.constant.CommonConstant;
import com.loit.common.utils.StringUtils;
import com.loit.loitcasclient.utils.ClientSpringContextUtil;
import com.loit.loitcasclient.utils.ClientTokenUtil;
import com.loit.loitcasclient.utils.ClientUserUtil;

/**
 * @Description: 审计切面
 * @author: gzx
 * @date: 2019年10月26日 上午11:07:21
 * @version: V1.0
 */
@Aspect
@Component
public class AuditAspect {

	private Logger logger = LoggerFactory.getLogger(AuditAspect.class);
	@Autowired
	private JmsTemplate jmsTemplate;

	public static final Destination destination = new ActiveMQQueue(AuditConstants.AUDIT_MESSAGE);

	/**
	 * @param @param joinPoint 参数
	 * @return void 返回类型
	 * @throws
	 * @Description: 前置通知
	 * @author: gzx
	 * @date: 2019年10月26日 上午11:25:56
	 */
	@Before("@annotation(com.loit.common.annotation.BeforeAuditAnnotation)")
	public void doBefore(JoinPoint joinPoint) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("auditType", AuditConstants.AUDIT_TYPE_BEFORE);
		// 组装数据
		this.assemblyData(map, joinPoint);
		// 发送信息至mq，实现异步处理
		jmsTemplate.convertAndSend(destination, map);

	}

	/**
	 * @param @param joinPoint 参数
	 * @return void 返回类型
	 * @throws
	 * @Description: 后置通知
	 * @author: gzx
	 * @date: 2019年10月26日 上午11:43:28
	 */
	@AfterReturning(value = "@annotation(com.loit.common.annotation.AfterReturnAuditAnnotation)", returning = "returnValue")
	public void doAfterReturning(JoinPoint joinPoint, Object returnValue) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("auditType", AuditConstants.AUDIT_TYPE_AFTERRETURN);
		map.put("returnValue", returnValue);
		// 组装数据
		this.assemblyData(map, joinPoint);
		// 发送信息至mq，实现异步处理
		jmsTemplate.convertAndSend(destination, map);
		logger.info("****已经进入后置通知*****");
	}


	/**
	 * @param @param e 参数
	 * @return void 返回类型
	 * @throws
	 * @Description: 后置抛出异常
	 * @author: gzx
	 * @date: 2019年10月26日 下午3:52:58
	 */
	@AfterThrowing(value = "@annotation(com.loit.common.annotation.AfterThrowAuditAnnotation)", throwing = "e")
	public void afterThrowing(JoinPoint joinPoint, Exception e) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("auditType", AuditConstants.AUDIT_TYPE_AFTERTHROW);
		map.put("exceptionStr", e.getMessage());
		// 组装数据
		this.assemblyData(map, joinPoint);
		// 发送信息至mq，实现异步处理
		jmsTemplate.convertAndSend(destination, map);
	}

	private void assemblyData(Map<String, Object> map, JoinPoint joinPoint) {
		//用的最多 通知的签名
		Signature signature = joinPoint.getSignature();
		map.put("className", signature.getDeclaringTypeName());
		map.put("method", signature.getName());

		RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
		//从获取RequestAttributes中获取HttpServletRequest的信息
		HttpServletRequest request = (HttpServletRequest) requestAttributes.resolveReference(RequestAttributes.REFERENCE_REQUEST);
		//如果要获取Session信息的话，可以这样写：
		//HttpSession session = (HttpSession) requestAttributes.resolveReference(RequestAttributes.REFERENCE_SESSION);
		//获取请求参数
//		Enumeration<String> enumeration = request.getParameterNames();
//		Map<String, String> parameterMap = Maps.newHashMap();
//		while (enumeration.hasMoreElements()) {
//			String parameter = enumeration.nextElement();
//			parameterMap.put(parameter, request.getParameter(parameter));
//		}
//		String str = JSON.toJSONString(parameterMap);
//		map.put("args", str);
		Object[] parameterValues = joinPoint.getArgs();
		String args = "";
		if (parameterValues != null && parameterValues.length > 0) {
			args = JSON.toJSONString(parameterValues[0]);
		}
		map.put("args", args);
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		map.put("actDate", sdf.format(new Date()));
		//获取userId
		String userId = request.getHeader(CommonConstant.AUTHORIZATION);
//		String accessToken = ClientTokenUtil.cookieAccessToken();
//		if (StringUtils.isNotEmpty(accessToken)) {
//			ClientUserUtil clientUserUtil = (ClientUserUtil) ClientSpringContextUtil.getBean("clientUserUtil");
//			userId = clientUserUtil.getUserId(accessToken);
//		}
		map.put("userId", userId);
	}
}
