package com.loit.common.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.persistence.CrudDao;
import com.loit.common.persistence.DataEntity;
import com.loit.common.persistence.Page;

/**
 * Service基类
 * @author loit
 * @version 2014-05-16
 */
public abstract class CrudService<D extends CrudDao<T>, T extends DataEntity<T>> extends BaseService {
	
	/**
	 * 持久层对象
	 */
	@Autowired
	protected D dao;
	
	/**
	 * 获取单条数据
	 * @param id
	 * @return
	 */
	public T get(String id) {
		return (T) dao.get(id);
	}
	
	
	/**
	 * 获取单条数据
	 * @param id
	 * @return
	 */
	public T get(int id) {
		return (T) dao.get(id);
	}
	
	/**
	 * 获取单条数据
	 * @param entity
	 * @return
	 */
	public T get(T entity) {
		return (T) dao.get(entity);
	}
	
	/**
	 * 查询列表数据
	 * @param entity
	 * @return
	 */
	public List<T> findList(T entity) {
		List<T> list = dao.findList(entity);
		return list;
	}

	/**
	 * 查询所有列表数据
	 * @param entity
	 * @return
	 */
	public List<T> findAllList(T entity) {
		List<T> list = dao.findAllList(entity);
		return list;
	}
	
	/**
	 * 查询分页数据
	 * @param page 分页对象
	 * @param entity
	 * @return
	 */
	public Page<T> findPage(Page<T> page, T entity) {
		entity.setPage(page);
		page.setList(findList(entity));
		return page;
	}

	/**
	 * 保存数据（插入或更新）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void save(T entity,Integer id) {
		if (id == null){
			entity.preInsert();
			dao.insert(entity);
		}else{
			entity.preUpdate();
			dao.update(entity);
		}
	}
	
	/**
	 * 保存数据（插入）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void insert(T entity) {
		entity.preInsert();
		dao.insert(entity);		
	}
	
	/**
	 * 保存数据（更新）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void update(T entity) {
		entity.preUpdate();
		dao.update(entity);		
	}
	
	/**
	 * 删除数据（物理删除）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void delete(T entity) {
		dao.delete(entity);
	}
	
	/**
	 * 删除数据（逻辑删除）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void deleteByLogic(T entity) {
		dao.deleteByLogic(entity);
	}
	
	
	/**
	 * 删除全部数据（物理删除）
	 * @param ids
	 */
	@Transactional(readOnly = false)
	public void deleteAll(String ids,T entity) {
		String idArray[] =ids.split(",");
        for(String id : idArray){
            entity = get(id);
            dao.delete(entity);
        }
	}
	
	/**
	 * 删除全部数据（逻辑删除）
	 * @param ids
	 */
	@Transactional(readOnly = false)
	public void deleteAllByLogic(String ids,T entity) {
		String idArray[] =ids.split(",");
        for(String id : idArray){
            entity = get(id);
            dao.deleteByLogic(entity);
        }
	}

	
	/**
	 * 获取单条数据
	 * @param id
	 * @return
	 */
	public T findUniqueByProperty(String propertyName, Object value){
		return dao.findUniqueByProperty(propertyName, value);
	}
	
}
