/**
 * Copyright &copy; 2015-2020 isaac All rights reserved.
 */
package com.loit.common.utils;

import org.apache.commons.lang3.time.DateFormatUtils;

import java.sql.Time;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 日期工具类, 继承org.apache.commons.lang.time.DateUtils类
 * @author loit
 * @version 2014-4-15
 */
@SuppressWarnings({ "unchecked", "rawtypes","static-access" })
public class DateUtils extends org.apache.commons.lang3.time.DateUtils {
	
	private static String[] parsePatterns = {
		"yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
		"yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
		"yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};

	/**
	 * 得到当前日期字符串 格式（yyyy-MM-dd）
	 */
	public static String getDate() {
		return getDate("yyyy-MM-dd");
	}
	
	/**
	 * 得到当前日期字符串 格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
	 */
	public static String getDate(String pattern) {
		return DateFormatUtils.format(new Date(), pattern);
	}
	
	/**
	 * 得到日期字符串 默认格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
	 */
	public static String formatDate(Date date, Object... pattern) {
		String formatDate = null;
		if (pattern != null && pattern.length > 0) {
			formatDate = DateFormatUtils.format(date, pattern[0].toString());
		} else {
			formatDate = DateFormatUtils.format(date, "yyyy-MM-dd");
		}
		return formatDate;
	}
	
	/**
	 * 得到日期时间字符串，转换格式（yyyy-MM-dd HH:mm:ss）
	 */
	public static String formatDateTime(Date date) {
		return formatDate(date, "yyyy-MM-dd HH:mm:ss");
	}

	/**
	 * 得到当前时间字符串 格式（HH:mm:ss）
	 */
	public static String getTime() {
		return formatDate(new Date(), "HH:mm:ss");
	}

	/**
	 * 得到当前日期和时间字符串 格式（yyyy-MM-dd HH:mm:ss）
	 */
	public static String getDateTime() {
		return formatDate(new Date(), "yyyy-MM-dd HH:mm:ss");
	}

	/**
	 * 得到当前年份字符串 格式（yyyy）
	 */
	public static String getYear() {
		return formatDate(new Date(), "yyyy");
	}

	/**
	 * 得到当前月份字符串 格式（MM）
	 */
	public static String getMonth() {
		return formatDate(new Date(), "MM");
	}

	/**
	 * 得到当天字符串 格式（dd）
	 */
	public static String getDay() {
		return formatDate(new Date(), "dd");
	}

	/**
	 * 得到当前星期字符串 格式（E）星期几
	 */
	public static String getWeek() {
		return formatDate(new Date(), "E");
	}
	
	/**
	 * 日期型字符串转化为日期 格式
	 * { "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", 
	 *   "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm",
	 *   "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm" }
	 */
	public static Date parseDate(Object str) {
		if (str == null){
			return null;
		}
		try {
			return parseDate(str.toString(), parsePatterns);
		} catch (ParseException e) {
			return null;
		}
	}

	/**
	 * 获取过去的天数
	 * @param date
	 * @return
	 */
	public static long pastDays(Date date) {
		long t = new Date().getTime()-date.getTime();
		return t/(24*60*60*1000);
	}

	/**
	 * 获取过去的小时
	 * @param date
	 * @return
	 */
	public static long pastHour(Date date) {
		long t = new Date().getTime()-date.getTime();
		return t/(60*60*1000);
	}
	
	/**
	 * 获取过去的分钟
	 * @param date
	 * @return
	 */
	public static long pastMinutes(Date date) {
		long t = new Date().getTime()-date.getTime();
		return t/(60*1000);
	}
	
	/**
	 * 转换为时间（天,时:分:秒.毫秒）
	 * @param timeMillis
	 * @return
	 */
    public static String formatDateTime(long timeMillis){
		long day = timeMillis/(24*60*60*1000);
		long hour = (timeMillis/(60*60*1000)-day*24);
		long min = ((timeMillis/(60*1000))-day*24*60-hour*60);
		long s = (timeMillis/1000-day*24*60*60-hour*60*60-min*60);
		long sss = (timeMillis-day*24*60*60*1000-hour*60*60*1000-min*60*1000-s*1000);
		return (day>0?day+",":"")+hour+":"+min+":"+s+"."+sss;
    }
	
	/**
	 * 获取两个日期之间的天数
	 * 
	 * @param before
	 * @param after
	 * @return
	 */
	public static double getDistanceOfTwoDate(Date before, Date after) {
		long beforeTime = before.getTime();
		long afterTime = after.getTime();
		return (afterTime - beforeTime) / (1000 * 60 * 60 * 24);
	}
	/**
	 * 获取两个日期之间的时差
	 * 
	 * @param before
	 * @param after
	 * @return
	 */
	public static double getDistanceOfTwoHour(Date before, Date after) {
		long nd = 1000 * 24 * 60 * 60;
		long nh = 1000 * 60 * 60;
		long beforeTime = before.getTime();
		long afterTime = after.getTime();
		return (afterTime - beforeTime) % nd/ nh;
	}
	
	
	/**
	 * 获取两个日期之间月份差
	 * 
	 * @param before
	 * @param after
	 * @return
	 */
	public static double getDistanceOfTwoMonth(Date before, Date after) {
		int result = 0;
		Calendar c1 = Calendar.getInstance();
		Calendar c2 = Calendar.getInstance();
		c1.setTime(before);
		c2.setTime(after);
		result = c2.get(Calendar.MONDAY) - c1.get(Calendar.MONTH);
		return result == 0 ? 1 : Math.abs(result);
	}
	/**
	 * 
	 * @param resourceDate 要进行加减的源时间点
	 * @param dateType Calendar.MONTH，月。。。
	 * @param time
	 * @return
	 * author lvhp
	 * date 2017-11-17
	 */
	public static Date addDate(Date resourceDate,int dateType,int time){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(resourceDate);
		calendar.add(dateType, time);
		try {
			return parseDate(formatDateTime(calendar.getTime()),"yyyy-MM-dd hh:mm:ss");
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return null;
	}
	

	/**
	 * 根据当前日期 求过去或未来天数 的日期
	 * @param past 日
	 * @param year 年
	 * @return
	 * author lishuai
	 */
	 public static String getPastDate(int past,int year) {
         Calendar calendar = Calendar.getInstance();
         calendar.set(Calendar.DAY_OF_YEAR, calendar.get(Calendar.DAY_OF_YEAR) - past);
         if(year==-1){
        	 calendar.add(calendar.YEAR, year);
         }
         Date today = calendar.getTime();
         SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
         String result = format.format(today);
         
         return result;
     }
	/**
	 * 	//获取当前(上，下)周的日期范围如：...,-1上一周，0本周，1下一周...  
	 * @param i
	 * @return
	 * author lishuai
	 */
    public static Map getWeekDays(int i) {  
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");   
        Calendar calendar1 = Calendar.getInstance();  
        calendar1.setFirstDayOfWeek(Calendar.MONDAY);  
          
        // 判断要计算的日期是否是周日，如果是则减一天计算周六的，否则会出问题，计算到下一周去了  
        int dayOfWeek = calendar1.get(Calendar.DAY_OF_WEEK);// 获得当前日期是一个星期的第几天  
        if (1 == dayOfWeek) {  
            calendar1.add(Calendar.DAY_OF_MONTH, -1);  
        }  
  
        int day = calendar1.get(Calendar.DAY_OF_WEEK);  
        calendar1.add(Calendar.DATE, 7*i);  
        calendar1.add(Calendar.DATE, calendar1.getFirstDayOfWeek() - day);  
        String beginDate = sdf.format(calendar1.getTime());  
        calendar1.add(Calendar.DATE, 6);  
      
        String endDate = sdf.format(calendar1.getTime());  
        Map map=new HashMap(); 
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
		return map;  
    } 
    /**
	 * 	//获取两个日期 之间的日期  
	 * @param i
	 * @return
	 * author lishuai
	 */
    public static List<Date> getBetweenDates(String firstday, String lastday) {
		try {
			SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd");
			Date start= null;
			start = sdf.parse(firstday);

			Date end=sdf.parse(lastday);
			List<Date> result = new ArrayList<Date>();
			result.add(start);
			Calendar tempStart = Calendar.getInstance();
			tempStart.setTime(start);
			tempStart.add(Calendar.DAY_OF_YEAR, 1);
			Calendar tempEnd = Calendar.getInstance();
			tempEnd.setTime(end);
			while (tempStart.before(tempEnd)) {
				result.add(tempStart.getTime());
				tempStart.add(Calendar.DAY_OF_YEAR, 1);
			}
			result.add(end);
			return result;
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}
	}

    /**
     * 当月 日期区间
     * @return
     * author lishuai
     */
    public  static Map getMonthDay(){
    	Calendar cale = Calendar.getInstance();
    	// 获取当月第一天和最后一天
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String beginDate, endDate;
        // 获取前月的第一天
        cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 0);
        cale.set(Calendar.DAY_OF_MONTH, 1);
        beginDate = format.format(cale.getTime());
        // 获取前月的最后一天
        cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 1);
        cale.set(Calendar.DAY_OF_MONTH, 0);
        endDate = format.format(cale.getTime());
        Map map=new HashMap(); 
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
		return map;
    }
    /**
     * 综合执法三高用的
     * @return
     */
    public  static Map get(){
    	Calendar cale = Calendar.getInstance();
    	// 获取当月第一天和最后一天
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String beginDate, endDate;
        // 获取前月的第一天
        cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 0);
        cale.set(Calendar.DAY_OF_MONTH, 1);
        beginDate = format.format(cale.getTime());
        // 获取前月的最后一天
        cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 1);
        cale.set(Calendar.DAY_OF_MONTH, 1);
        endDate = format.format(cale.getTime());
        Map map=new HashMap(); 
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
		return map;
    }
    
    
    /** 
     * 获取上个月的时间范围 
     * @return 
     */  
    public static Map getLastMonth(){  
        Calendar startCalendar = Calendar.getInstance();  
        startCalendar.add(Calendar.MONTH, -1);  
        startCalendar.set(Calendar.DAY_OF_MONTH, 1);  
        setMinTime(startCalendar);  
          
        Calendar endCalendar = Calendar.getInstance();  
        endCalendar.add(Calendar.MONTH, -1);  
        endCalendar.set(Calendar.DAY_OF_MONTH, endCalendar.getActualMaximum(Calendar.DAY_OF_MONTH));  
        setMaxTime(endCalendar);
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd");  
         
        String beginDate=sdf.format(startCalendar.getTime());  
        String endDate=sdf.format(endCalendar.getTime()); 
        Map map=new HashMap(); 
        map.put("beginDate", beginDate);
        map.put("endDate", endDate); 
        return  map;  
    } 
   
    	/**
    	 * 去年当日日期
    	 * @param year yyyy-MM-dd
    	 * @return
    	 * @throws ParseException
    	 * author lishuai
    	 */
    public static String getOldYear(String year) throws ParseException{
    	SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		Calendar c = Calendar.getInstance();
		c.setTime(format.parse(year));
        c.add(Calendar.YEAR, -1);
        Date y = c.getTime();
        return format.format(y);
    }
    
    private static void setMinTime(Calendar calendar){  
        calendar.set(Calendar.HOUR_OF_DAY, 0);  
        calendar.set(Calendar.MINUTE, 0);  
        calendar.set(Calendar.SECOND, 0);  
        calendar.set(Calendar.MILLISECOND, 0);  
    } 
    private static void setMaxTime(Calendar calendar){  
        calendar.set(Calendar.HOUR_OF_DAY, calendar.getActualMaximum(Calendar.HOUR_OF_DAY));  
        calendar.set(Calendar.MINUTE, calendar.getActualMaximum(Calendar.MINUTE));  
        calendar.set(Calendar.SECOND, calendar.getActualMaximum(Calendar.SECOND));  
        calendar.set(Calendar.MILLISECOND, calendar.getActualMaximum(Calendar.MILLISECOND));  
    }  
    
    
    /**
	 * <li>比较两个时间类型的大小</li>
	 * @param date
	 * @return  第一个日期大返回 true  
	 */
	public static boolean compareDate(Time time1, Time time2) {
		boolean flag = false;
		flag = time1.getTime()> time2.getTime();		
		return flag;
	}
	
	 /**
		 * <li>比较两个时间类型的大小</li>
		 * @param date
		 * @return  第一个日期大返回 true  
		 */
		public static boolean compareDate(String time1, Date date2 ) {
			boolean flag = false;
			SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss");//创建日期转换对象HH:mm:ss为时分秒，年月日为yyyy-MM-dd  
			Date temp;
			try {
				temp = df.parse(df.format(date2));
				flag = parseDate(time1,"HH:mm:ss").getTime()> temp.getTime();
			} catch (ParseException e) {
				e.printStackTrace();
			}
			
			return flag;
		}
	
	/**
	 * @param args
	 * @throws ParseException
	 */
	/*public static void main(String[] args) throws ParseException {
		//Date date = DateUtils.addDate(new Date(), Calendar.MINUTE, 10);
//		getLastMonth();
		//SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss");
		//boolean flag = DateUtils.compareDate(new Time(df.parse("11:41:00").getTime()),new Time(df.parse("13:39:00").getTime()));
		
		System.out.println("本周"+ThisWeek());
		System.out.println("上周"+LastWeek());
		System.out.println("去年本周"+LastYearWeek());
	}*/
	public static List ThisWeek(){
		List list = new ArrayList();
		Calendar cal = Calendar.getInstance();
		int date = cal.get(Calendar.DAY_OF_MONTH);
		int n = cal.get(Calendar.DAY_OF_WEEK);
		if (n == 1) {
		n = 7;
		} else {
		n = n - 1;
		}
		//System.out.println("当天为本周第" + n + "天");
		// 日期格式
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		for (int i = 1; i <= 8; i++) {
		cal.set(Calendar.DAY_OF_MONTH, date + i - n);
		//System.out.println("本周第" + i + "天：" + sdf.format(cal.getTime()));
		list.add(sdf.format(cal.getTime())+" 00:00");
		}
		return list;
	}
	
	public static List LastWeek(){
		List list = new ArrayList();
		// 日期格式转换
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        // 当前日期
        Calendar instance = Calendar.getInstance();
        // 调整到上周
        instance.add(Calendar.WEDNESDAY, -1);
        // 调整到上周1
        instance.set(Calendar.DAY_OF_WEEK, 1);
        //循环打印
        for (int i = 0; i <= 7; i++) {
            //System.out.println("星期" + i + ":" + format.format(instance.getTime()));
            instance.add(Calendar.DAY_OF_WEEK, 1);
            list.add(format.format(instance.getTime())+" 00:00");
        }
        return list;
	}
	
	public static List LastYearWeek(){
		List list = new ArrayList();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.YEAR, -1);
		int date = cal.get(Calendar.DAY_OF_MONTH);
		int n = cal.get(Calendar.DAY_OF_WEEK);
		if (n == 1) {
		n = 7;
		} else {
		n = n - 1;
		}
		//System.out.println("当天为本周第" + n + "天");
		// 日期格式
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		for (int i = 1; i <= 8; i++) {
		cal.set(Calendar.DAY_OF_MONTH, date + i - n);
		//System.out.println("本周第" + i + "天：" + sdf.format(cal.getTime()));
		list.add(sdf.format(cal.getTime())+" 00:00");
		}
		return list;
	}

	/**
	 * @Description 判断日期在当天之前还是之后
	 * @Author NJ
	 * @Date 2020/1/2  14:25
	 * @Param
	 * @Return -1大于当天 1小于等于当天
	*/
	public static boolean compareDateAfterDay(Date date) {
		Boolean result=false;
		SimpleDateFormat sdf = new SimpleDateFormat(parsePatterns[0]);
		try {
			Integer a = new Date().compareTo(sdf.parse(sdf.format(date)));
			if (-1==a) {
				result= true;
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return result;

	}

}
