/**
 * Copyright &copy; 2015-2020 isaac All rights reserved.
 */
package com.loit.modules.sys.interceptor;

import com.loit.common.config.Global;
import com.loit.common.constant.CommonConstant;
import com.loit.common.service.BaseService;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.CookieUtils;
import com.loit.common.utils.StringUtils;
import org.slf4j.MDC;
import org.springframework.core.NamedThreadLocal;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.InetAddress;
import java.text.SimpleDateFormat;

/**
 * 日志拦截器
 * @author zpc
 * @version 2014-8-19
 */
public class LogInterceptor extends BaseService implements HandlerInterceptor {

	private final static String USER_ID = "userId";
	private final static String QUERY_STRING = "queryString";
	private final static String HOST_IP = "hostIp";
	private final static String HOST_PORT = "hostPort";
	private final static String REQ_URI="reqUri";
	private final static String REQ_TYPE="reqType";
    private static AntPathMatcher ant = new AntPathMatcher();

	private static final ThreadLocal<Long> startTimeThreadLocal =
			new NamedThreadLocal<Long>("ThreadLocal StartTime");
	
	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, 
			Object handler) throws Exception {
		long beginTime = System.currentTimeMillis();//1、开始时间
		insertIntoMDC(request);
		if (logger.isDebugEnabled()){
			if(matchUri(request.getRequestURI())) {
				return true;
			}
	        startTimeThreadLocal.set(beginTime);		//线程绑定变量（该数据只有当前请求的线程可见）  
	        logger.debug("[{}] 开始计时: {}  URI: {} ", beginTime, new SimpleDateFormat("hh:mm:ss.SSS")
	        	.format(beginTime), request.getRequestURI());
		}
		return true;
	}

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, 
			ModelAndView modelAndView) throws Exception {
		if (modelAndView != null){
			logger.info("ViewName: " + modelAndView.getViewName());
		}
		clearMDC();
	}
	
	public boolean matchUri(String uri) {
		if(ant.match("/image/**", uri)||ant.match("/static/**", uri)||"/error".equals(uri)||"/".equals(uri)) {
			return true;
		}
		return false;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response, 
			Object handler, Exception ex) throws Exception {

		// 保存日志
		//LogUtils.saveLog(request, handler, ex, null);
		
		// 打印JVM信息。
		if (logger.isDebugEnabled()){
			if(matchUri(request.getRequestURI())) {
				return;
			}
			long beginTime = startTimeThreadLocal.get();//得到线程绑定的局部变量（开始时间）  
			long endTime = System.currentTimeMillis(); 	//2、结束时间  
			long timeSlot = endTime - beginTime;
			String timeDesc = "time consuming";
			if(timeSlot>1000&&timeSlot<= 5000) {
				timeDesc = "long time consuming";
			}else if(timeSlot>5000) {
				timeDesc = "super long time consuming";
			}
	        logger.debug("[{}] 计时结束: {}  URI: {}  {}: {}  最大内存: {}m  已分配内存: {}m  已分配内存中的剩余空间: {}m  最大可用内存: {}m",
	        		beginTime,new SimpleDateFormat("hh:mm:ss.SSS").format(endTime), request.getRequestURI(), timeDesc , timeSlot+"ms",
					 Runtime.getRuntime().maxMemory()/1024/1024, Runtime.getRuntime().totalMemory()/1024/1024, Runtime.getRuntime().freeMemory()/1024/1024, 
					(Runtime.getRuntime().maxMemory()-Runtime.getRuntime().totalMemory()+Runtime.getRuntime().freeMemory())/1024/1024); 
		}
	}


	/**
	 * @Description: TODO(将请求相关信息存放到MDC中，以便在logback日志中输出)
	 * @author yangwenbin
	 * @date
	 */
	void insertIntoMDC(HttpServletRequest request) {
		//获取服务IP
		String hostIp = Global.HOSTIP;
		//获取服务端口
		String hostPort = String.valueOf(request.getLocalPort());
		//获取参数
		String queryString = request.getQueryString();
		//请求地址
		String reqUri = request.getRequestURI();
		//请求方式
		String reqType = request.getMethod();
		String accessToken = CookieUtils.getCookie(request, CommonConstant.TOKEN_NAME);
		String userId = "";
		//用户ID
		if(StringUtils.isNotEmpty(accessToken)){
			userId = (String) CacheUtils.get(accessToken);
		}
		//以下是存放相关信息到MDC中
		MDC.put(HOST_IP,hostIp);
		MDC.put(HOST_PORT,hostPort);
		MDC.put(QUERY_STRING,queryString);
		MDC.put(REQ_URI,reqUri);
		MDC.put(REQ_TYPE,reqType);
		MDC.put(USER_ID,userId);


	}
	/**
	 * @Description: TODO(获取服务IP地址)
	 * @author yangwenbin
	 * @date
	 */
	public String getHostIp() {
		try {
			InetAddress address = InetAddress.getLocalHost();
			return address.getHostAddress();

		} catch (Exception e) {
			logger.error("获取服务IP地址异常{}",e);
		}
		return null;
	}
	/**
	 * @Description: TODO(返回结果之前，清除MDC存放的值)
	 * @author yangwenbin
	 * @date
	 */
	void clearMDC(){
		MDC.remove(HOST_IP);
		MDC.remove(HOST_PORT);
		MDC.remove(QUERY_STRING);
		MDC.remove(REQ_URI);
		MDC.remove(REQ_TYPE);
		MDC.remove(USER_ID);
	}

}
