package com.loit.getway.utils;

import java.io.Serializable;
import java.util.*;
import java.util.concurrent.TimeUnit;

import javax.annotation.Resource;

import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import org.springframework.stereotype.Component;

/**
 * redicache 工具类
 * 
 */
@Component
@SuppressWarnings("unchecked")
public class RedisUtil {
	
	@SuppressWarnings("rawtypes")
	private static RedisTemplate redisTemplate;
	@SuppressWarnings("rawtypes")
	@Resource
	public void setRedisTemplate(RedisTemplate redisTemplate){
		//用于解决模糊匹配时无法匹配全部的key值，故把key去掉序列化
		RedisSerializer stringSerializer = new StringRedisSerializer();
		redisTemplate.setKeySerializer(stringSerializer);
		this.redisTemplate = redisTemplate;
	}
	/**
	 * 批量删除对应的value
	 * 
	 * @param keys
	 */
	public static void remove(final String... keys) {
		for (String key : keys) {
			remove(key);
		}
	}
	/**
	 * 批量删除key
	 * 
	 * @param pattern
	 */
	public static void removePattern(final String pattern) {
		Set<Serializable> keys = redisTemplate.keys(pattern);
		if (keys.size() > 0)
		redisTemplate.delete(keys);
	}
	/**
	 * 删除对应的value
	 * 
	 * @param key
	 */
	public static void remove(final String key) {
		if (exists(key)) {
			redisTemplate.delete(key);
		}
	}
	/**
	 * 判断缓存中是否有对应的value
	 * 
	 * @param key
	 * @return
	 */
	public static boolean exists(final String key) {
		return redisTemplate.hasKey(key);
	}
	/**
	 * 读取缓存
	 * 
	 * @param key
	 * @return
	 */
	public static Object get(final String key) {
		Object result = null;
		ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
		result = operations.get(key);
		return result;
	}
	/**
	 * 写入缓存
	 * 
	 * @param key
	 * @param value
	 * @return
	 */
	public static boolean set(final String key, Object value) {
		boolean result = false;
		try {
			ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
			operations.set(key, value);
			result = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}
	/**
	 * 写入缓存
	 * 有效期
	 * @param key
	 * @param value
	 * @param expireTime 时效，单位：秒
	 * @return
	 */
	public static boolean set(final String key, Object value, Long expireTime) {
		boolean result = false;
		try {
			ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
			operations.set(key, value);
			redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
			result = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 新增一个有序集合
	 * @param key
	 * @param value
	 * @param expireTime
	 */
	public static void zAdd(final String key, String value, Long expireTime) {
		try {
			ZSetOperations<String, String> operations = redisTemplate.opsForZSet();
			operations.add(key, value, expireTime);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 删除一个有序集合
	 * @param key
	 * @param value
	 */
	public static void zRemove(final String key, String value) {
		try {
			ZSetOperations<String, String> operations = redisTemplate.opsForZSet();
			operations.remove(key, value);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 根据指定的score值得范围来移除成员
	 * @param key
	 * @param min
	 * @param max
	 */
	public static void removeRangeByScore(final String key, Double min, Double max) {
		try {
			ZSetOperations<String, String> operations = redisTemplate.opsForZSet();
			operations.removeRangeByScore(key, min, max);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 获取有序集合的个数
	 * @param key
	 * @return
	 */
	public static Long zCard(final String key) {
		Long count = 0L;
		try {
			ZSetOperations<String, String> operations = redisTemplate.opsForZSet();
			count = operations.zCard(key);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return count;
	}

	/**
	 * 获取有序集合，按分数值递增(从小到大)顺序排列
	 * @param key
	 * @return
	 */
	public static Set<String> zRange(final String key) {
		// 删除过了有效期的数据
		removeRangeByScore(key, 0.0, Double.valueOf(System.currentTimeMillis()));

		Set<String> list = new HashSet<>();
		try {
			ZSetOperations<String, String> operations = redisTemplate.opsForZSet();
			list = operations.range(key, 0, -1);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}
}
