/**
 * Copyright &copy; 2015-2020 isaac All rights reserved.
 */
package com.loit.modules.sys.utils;

import javax.servlet.http.HttpServletRequest;

import com.loit.common.constant.LoitHeaderConstant;
import com.loit.common.spring.timeloit.utils.LogUtil;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.RedisConstants;
import com.loit.common.utils.*;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.service.OrgService;
import com.loit.v1.modules.service.RoleService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import com.loit.loitcasclient.utils.ClientSpringContextUtil;
import com.loit.loitcasclient.utils.ClientTokenUtil;
import com.loit.loitcasclient.utils.ClientUserUtil;
import com.loit.modules.sys.entity.User;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.dao.AccoutInfoDao;
import com.loit.v1.modules.entity.AccoutInfo;
import java.util.List;

/**
 * 用户工具类
 * 
 * @author gzx
 * @version 2019-11-04
 */
public class UserUtils {
	private static Logger logger = LoggerFactory.getLogger(UserUtils.class);
	private static AccoutInfoDao accoutInfoDao;
	private static RoleService roleService;
	private static OrgService orgService;

	public static final String USER_CACHE = "userCache";
	
    public static AccoutInfoDao getAccoutInfoDao() {
        if(accoutInfoDao==null) {
        	accoutInfoDao = SpringContextHolder.getBean(AccoutInfoDao.class);
        }
        return accoutInfoDao;
    }

	public static RoleService getRoleService() {
		if(roleService==null) {
			roleService = SpringContextHolder.getBean(RoleService.class);
		}
		return roleService;
	}

	public static OrgService getOrgService() {
    	if(orgService == null){
			orgService = SpringContextHolder.getBean(OrgService.class);
		}
		return orgService;
	}

	/**
     * 获取当前登录用户
	 *
	 * 1.cookie或request中获取令牌
	 * 2.用令牌从cache中获取userId
	 * 3.userId换User对象
	 *
	 * DEMON modify by 2019-11-19
     */
    public static User getUser(){
		// 获取request
		ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		if(servletRequestAttributes == null) {
			return null;
		}
		HttpServletRequest request = servletRequestAttributes.getRequest();

		// 开发环境多用户切换使用， PersonController.setDevCookie()设置userId
		String userId = CookieUtils.getCookie(request, LoitHeaderConstant.AUTHORIZATION);
		if(StringUtils.isNotEmpty(userId)){
			return get(userId);
		}

		// 每次请求时，网关使用令牌去登录服务器获取userId并放入Header中
		userId = request.getHeader(LoitHeaderConstant.AUTHORIZATION);
		if(StringUtils.isNotEmpty(userId)){
			return get(userId);
		}

		// 没走网关的第三方应用从cookie中获取令牌
		String accessToken = ClientTokenUtil.cookieAccessToken();
		if(StringUtils.isNotEmpty(accessToken)){
			// 页面 access token 换 userId
			ClientUserUtil clientUserUtil = (ClientUserUtil) ClientSpringContextUtil.getBean("clientUserUtil");
			userId = clientUserUtil.getUserId(accessToken);
			if(StringUtils.isNotEmpty(userId)){
				return get(userId);
			}
		}

		return null;
    }

	/**
	 * 根据ID获取用户缓存信息
	 *
	 * @param userId  账号ID
	 * @return 取不到返回null
	 */
	public static User get(String userId) {
		String key = new StringReplaceUtil(RedisConstants.PORTAL_USER_KEY).replace("userId",  userId).toString();
		boolean sign = true;
		User user = null;
		try {
			user = (User) CacheUtils.get(USER_CACHE, key);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(LogUtil.print("errorCode:{},info:用户工具类，{}"), LoitStatusCode.REDIS_LINK_FAIL.statusCode,LoitStatusCode.REDIS_LINK_FAIL.statusDesc);
			sign = false;
		}
		if (user == null) {
			// 获取用户基本信息
			user = getAccoutInfoDao().getCacheUserInfo(userId);
			if (user == null) {
				return null;
			}

			// 获取用户权限列表
			List<Role> roles = getRoleService().findByAccountId(user.getAccountId());
			if(ListUtil.isNotEmpty(roles)){
				user.setRoleList(roles);
			}

			// 获取机构id， 查出一堆机构，最后一个就是当前人所在部门的上级机构
			List<Org> orgs = getOrgService().findOryByOrgLevelCode(user.getOrgLevelCode());
			if(ListUtil.isNotEmpty(orgs)){
                Org org = orgs.get(orgs.size() - 1);
                user.setSuperOrgId(org.getOrgId());
                user.setSuperOrgName(org.getOrgName());
                user.setSuperOrgShortName(org.getOrgShortName());
				user.setSuperOrgExtCode(org.getExtCode());
			}

			if(sign) {
				CacheUtils.put(USER_CACHE, key, user);
			}
		}
		return user;
	}
    
    /**
	 * 清除指定用户缓存
	 * @param user
	 */
	public static void clearCache(User user){
		String key = new StringReplaceUtil(RedisConstants.PORTAL_USER_KEY).replace("userId",  user.getUserId()).toString();
		CacheUtils.remove(USER_CACHE, key);
	}
    
    /**
    * @Description: 获取用户详细信息
    * @author: gzx
    * @date: 2019年11月4日 下午4:20:54 
    * @param @param userId
    * @param @return 参数 
    * @return AccoutInfo 返回类型 
    * @throws
     */
    public PersonBaseDTO getAccountInfo(String userId) {
    	AccoutInfo accoutInfo = new AccoutInfo();
    	accoutInfo.setUserId(userId);
    	return getAccountInfo(accoutInfo);
    }
    /**
    * @Description: 获取用户详细信息
    * @author: gzx
    * @date: 2019年11月4日 下午4:29:50 
    * @param @param accoutInfo
    * @param @return 参数 
    * @return PersonBaseDTO 返回类型 
    * @throws
     */
    public PersonBaseDTO getAccountInfo(AccoutInfo accoutInfo) {
    	return accoutInfoDao.getInfo(accoutInfo);
    }

}
