package com.loit.modules.sys.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.loit.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Dict;
import com.loit.modules.sys.service.DictService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;
/**
 * @description 字典controller
 * @author: gzx
 * @date: 2019年11月4日 上午9:40:29
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/dict")
@Api(value = "DictController", tags = {"系统字典Controller"})
public class DictController {
	protected Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private DictService dictService;
	
	/**
	 * @description 系统字典分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 系统字典实体类
	 * @param request
	 * @param response
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="分页列表", notes="获取系统字典分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictCode", value = "字典编码", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictName", value = "字典名称", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "baseFlag", value = "1、系统内置 2、用户自定义 ", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson list(@ApiIgnore Dict dict, HttpServletRequest request, HttpServletResponse response) {
		Page<Dict> page = dictService.findTreePage(new Page<Dict>(request, response), dict);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}


	/**
	 * @description 系统字典详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dataId 系统字典ID
	 * @return
	 */
	@GetMapping(value = "{dictId}")
	@ApiOperation(value="详情", notes="根据dictId获取系统字典详情")
	@ApiImplicitParam(name = "dictId", value = "系统字典ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String dictId) {


		if(StringUtils.isBlank(dictId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		Dict dict = dictService.get(dictId);
		AjaxJson json = new AjaxJson();
		json.put("obj", dict);
		return json;
	}


	/**
	 * @description 系统字典新增或更新
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 
	 * @return
	 */
	@PostMapping(value = "")
	@ApiOperation(value = "保存(新增或修改)", notes = "保存或更新系统字典")
	@ApiImplicitParam(name = "dict", value = "字典实体类", required = true, dataType = "Dict")
	public AjaxJson save(@RequestBody Dict dict) {
		AjaxJson json = new AjaxJson();
		try {			
			List<Dict> list = dictService.findList(dict);
			if(list.size()>0) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.DICT_IS_EXIST);
			}else {
				dictService.save(dict);
				json.setMsg("字典保存成功");
			}			
			return json;
		} catch (Exception e) {
			logger.error("erroCode:{},info:保存字典数据异常",LoitStatusCode.FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}
	}


	/**
	 * @description 系统字典删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dictId 字典ID，多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除系统字典，多ID时，以逗号分隔")
	@DeleteMapping(value = "{dictId}")
	public AjaxJson deleteAll(@PathVariable("dictId") String dictId) {
		if(StringUtils.isBlank(dictId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			dictService.deleteAll(dictId);
			json.setMsg("删除字典成功");
			return json;
		} catch (Exception e) {
			logger.error("erroCode:{},info:删除字典异常",LoitStatusCode.REDIS_LINK_FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.REDIS_LINK_FAIL);
		}
	}
	
	/**
	 * @description 检查字典名称是否重复
	 * @param dictId
	 * @param dictName
	 * @return
	 */
	@ApiOperation(value = "检查字典名称是否重复", notes = "检查字典名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dictName", value = "字典名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkDictNameUnique")
	public AjaxJson checkDictNameUnique(Integer dictId,String dictName) {
		boolean uniqueFlag = dictService.checkNameUnique(dictId, dictName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("字典名称不重复");
		} else {
			json.setMsg("字典名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
}
