package com.loit.modules.sys.web;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.DictData;
import com.loit.modules.sys.service.DictDataService;
import com.loit.modules.sys.utils.DictDataUtils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @description 系统字典数据项Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/dictData")
@Api(value = "DictDataController", tags = {"系统字典数据项Controller"})
public class DictDataController extends BaseController {

	@Autowired
	private DictDataService dictDataService;

	/**
	 * @description 系统字典数据项分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 系统字典数据项实体类
	 * @param request
	 * @param response
	 * @return
	 */
	@GetMapping(value = "page")
	@ApiOperation(value="分页列表", notes="获取系统字典数据项分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "String", paramType = "query")		
	})	
	public AjaxJson page(@ApiIgnore DictData dict, HttpServletRequest request, HttpServletResponse response) {
		Page<DictData> page = dictDataService.findPage(new Page<DictData>(request, response), dict);
        //logger.debug("系统字典数据项列表分页信息："+page);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	
	/**
	 * @description 系统字典数据项列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 系统字典数据项实体类
	 * @param request
	 * @param response
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="列表", notes="获取系统字典数据项列表")
	@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "String", paramType = "query")
	public AjaxJson list(@ApiIgnore DictData dict, HttpServletRequest request, HttpServletResponse response) {
		List<DictData> list = dictDataService.findList(dict);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}


	/**
	 * @description 系统字典数据项详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dataId
	 * @return
	 */
	@GetMapping(value = "{dataId}")
	@ApiOperation(value="详情", notes="根据dataId获取系统字典数据项详情")
	@ApiImplicitParam(name = "dataId", value = "系统字典数据项ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String dataId) {
		if(StringUtils.isBlank(dataId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		DictData dictData = dictDataService.get(dataId);
		AjaxJson json = new AjaxJson();
		json.put("obj", dictData);
        //logger.debug("系统字典数据项详情："+dictData);
		return json;
	}


	/**
	 * @description 系统字典数据项新增或更新
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 
	 * @return
	 */
	@PostMapping(value = "")
	@ApiOperation(value = "保存", notes = "保存或更新系统字典数据项")
	@ApiImplicitParam(name = "dict", value = "系统字典数据项实体", required = true, dataType = "DictData")
	public AjaxJson save(@RequestBody DictData dict) {
		AjaxJson json = new AjaxJson();
		try {
			List<DictData> list = dictDataService.findAllList(dict);
			if(list.size()>0) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.DICTDATA_IS_EXIST);
			}else {
				dictDataService.save(dict);
				json.setMsg("保存成功");
				return json;
			}
		} catch (Exception e) {
			logger.error("erroCode:{},info:保存系统字典数据项异常",LoitStatusCode.FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}
	}


	/**
	 * @description 系统字典数据项删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除系统字典数据项，多ID时，以逗号分隔")
	@DeleteMapping(value = "{dataId}")
	public AjaxJson deleteAll(@PathVariable("dataId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			dictDataService.deleteAll(ids);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("erroCode:{},info:删除字典数据项异常",LoitStatusCode.REDIS_LINK_FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.REDIS_LINK_FAIL);
		}
	}
	
	/**
	 * @description 从缓存中获取某字典数据项列表，如果缓存中没有，则从数据库中获取
	 * @param dictId
	 * @return
	 */
	@GetMapping(value = "getDictDataByDictId")
	@ApiOperation(value="从缓存中获取某字典数据项列表", notes="从缓存中获取某字典数据项列表，如果缓存中没有，则从数据库中获取")
	@ApiImplicitParam(name = "dictCode", value = "字典dictCode", required = true, dataType = "String", paramType="query")
	public AjaxJson listDictDataByDictId(@ApiIgnore String dictCode) {
		List<DictData> dictDataList = DictDataUtils.getDictDataList(dictCode);
		AjaxJson json = new AjaxJson();
		json.put("dictDataList", dictDataList);
		return json;
	}
	
	/**
	 * @description 检查字典项名称是否重复
	 * @param dictId
	 * @param dataId
	 * @param dictName
	 * @return
	 */
	@ApiOperation(value = "检查字典项名称是否重复", notes = "检查字典项名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dataId", value = "字典项ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dataName", value = "字典项名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkDataNameUnique")
	public AjaxJson checkDataNameUnique(Integer dictId,Integer dataId,String dataName) {
		boolean uniqueFlag = dictDataService.checkNameUnique(dictId, dataId,dataName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("字典项名称不重复");
		} else {
			json.setMsg("字典项名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}

	

}
