package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.Dict;
import com.loit.modules.sys.entity.DictData;
import com.loit.modules.sys.service.DictDataService;
import com.loit.modules.sys.utils.DictDataUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @description 系统字典数据项Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/feign/dictData")
@Api(value = "FeignDictDataController", tags = {"Feign 系统字典数据项"})
public class FeignDictDataController extends BaseController {

	@Autowired
	private DictDataService dictDataService;

	/**
	 * @description 系统字典数据项分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "page")
	@ApiOperation(value="分页列表", notes="获取系统字典数据项分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "String", paramType = "query")		
	})	
	public AjaxJson page(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "dictId", required = true) Integer dictId) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();

		DictData dictData = new DictData();
		dictData.setDictId(dictId);

		Page<DictData> page = dictDataService.findPage(new Page<DictData>(request, response), dictData);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	
	/**
	 * @description 系统字典数据项列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "list")
	@ApiOperation(value="列表", notes="获取系统字典数据项列表")
	@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "String", paramType = "query")
	public AjaxJson list(
			@RequestParam(value = "dictId", required = true) Integer dictId) {
		DictData dictData = new DictData();
		dictData.setDictId(dictId);

		List<DictData> list = dictDataService.findList(dictData);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}

	/**
	 * @description 系统字典数据项详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dataId
	 * @return
	 */
	@GetMapping(value = "{dataId}")
	@ApiOperation(value="详情", notes="根据dataId获取系统字典数据项详情")
	@ApiImplicitParam(name = "dataId", value = "系统字典数据项ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=true) String dataId) {
		if(StringUtils.isBlank(dataId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		DictData dictData = dictDataService.get(dataId);
		AjaxJson json = new AjaxJson();
		json.put("obj", dictData);
        //logger.debug("系统字典数据项详情："+dictData);
		return json;
	}

	/**
	 * @description 从缓存中获取某字典数据项列表，如果缓存中没有，则从数据库中获取
	 * @return
	 */
	@GetMapping(value = "/getDictDataByDictId")
	@ApiOperation(value="从缓存中获取某字典数据项列表", notes="从缓存中获取某字典数据项列表，如果缓存中没有，则从数据库中获取")
	@ApiImplicitParam(name = "dictCode", value = "字典dictCode", required = true, dataType = "String", paramType="query")
	public AjaxJson listDictDataByDictId(
			@RequestParam(value = "dictCode", required = true) String dictCode) {
		List<DictData> dictDataList = DictDataUtils.getDictDataList(dictCode);
		AjaxJson json = new AjaxJson();
		json.put("dictDataList", dictDataList);
		return json;
	}
	
	/**
	 * @description 检查字典项名称是否重复
	 * @return
	 */
	@ApiOperation(value = "检查字典项名称是否重复", notes = "检查字典项名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dataId", value = "字典项ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dataName", value = "字典项名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkDataNameUnique")
	public AjaxJson checkDataNameUnique(
			@RequestParam(value = "dictId", required = true) Integer dictId,
			@RequestParam(value = "dataId", required = false) Integer dataId,
			@RequestParam(value = "dataName", required = true) String dataName) {
		boolean uniqueFlag = dictDataService.checkNameUnique(dictId, dataId,dataName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("字典项名称不重复");
		} else {
			json.setMsg("字典项名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}

	

}
