package com.loit.v1.audit.service;

import com.loit.common.constant.AuditConstants;
import com.loit.common.service.CrudService;
import com.loit.common.utils.SpringContextHolder;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.audit.dao.AuditLogDao;
import com.loit.v1.audit.entity.AuditLevel;
import com.loit.v1.audit.entity.AuditLog;
import com.loit.v1.audit.entity.AuditStrategy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * @ClassName: AuditLogService
 * @Description: 行为审核日志表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
@Transactional(readOnly = true)
public class AuditLogService extends CrudService<AuditLogDao, AuditLog> {
		
	@Autowired
	private AuditStrategyService auditStrategyService;
	@Autowired
	private AuditLevelService auditLevelService;


	/**
	 * @return 
	* @Description: 记录相关审计日志并根据警告类型完成相应警告
	* @author: gzx
	* @date: 2019年10月28日 上午8:46:45 
	* @param @param map 含有className、method、userId、actDate、args、opType等值
	* @return void 返回类型 
	* @throws
	 */
	@Transactional(readOnly = false)
	public void writeAuditLog(Map<String,Object> map) {
		//获取类名
		String className = map.get("className").toString();
    	className = className.substring(className.lastIndexOf(".")+1);
    	//获取方法名
    	String method = map.get("method").toString();
    	AuditStrategy auditStrategy = auditStrategyService.getStrategyByClassAndMethod(className, method);
    	if(auditStrategy==null) {
    		logger.debug("根据className和method无法获取审计策略，无需审计。className={},method={}",className,method);
    	}else {
    		//日志表数据
    		AuditLog auditLog = new AuditLog();
    		String auditType =  map.get("auditType").toString();
    		String userId = "";
    		String userName = "";
    		if (map.get("userId") != null) {
				userId = map.get("userId").toString();
				User user = UserUtils.get(userId);
				userName = user != null ? user.getPersonName() : "";
			}
    		String args =  map.get("args").toString();
    		String actDate =  map.get("actDate").toString();
    		auditLog.setActor(userId);
    		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    		try {
				auditLog.setActTime(sdf.parse(actDate));
			} catch (ParseException e) {
				logger.error("审计日志行为事件转换失败！");
				e.printStackTrace();
			}
    		String alarmSucess = "0";
    		AuditLevel auditLevel = auditLevelService.get(auditStrategy.getLevelId());
    		auditLog.setAppCode(auditStrategy.getAppCode());
    		auditLog.setAppName(auditStrategy.getAppName());
    		auditLog.setLevelId(auditStrategy.getLevelId());
    		auditLog.setActCodeId(auditStrategy.getActCodeId());
    		auditLog.setObjectTypeId(auditStrategy.getObjectTypeId());
    		auditLog.setCreateTime(new Date());
    		auditLog.setUpdateTime(new Date());
    		
    		//警告类型不为空
			if(auditLevel.getAlertId() != null) {
				
				//警告通知
				alarmSucess = doAlarm(auditLevel,userId,userName,className,method,sdf.format(auditLog.getActTime()));
			}
			auditLog.setActStatus(alarmSucess);
			String actMessage = new StringBuilder("用户:").append(userName).append(",userId:").append(userId).append("操作了类").
					append(className).append("中方法").append(method).append(",参数为：").append(args).toString();
			if(StringUtils.equals(auditType, AuditConstants.AUDIT_TYPE_BEFORE)) {
				actMessage = new StringBuilder("AuditBefore审计,").append(actMessage).toString();
			}else if(StringUtils.equals(auditType, AuditConstants.AUDIT_TYPE_AFTERRETURN)) {
				actMessage = new StringBuilder("AuditAfterReturn审计,").append(actMessage).toString();
			}else if(StringUtils.equals(auditType, AuditConstants.AUDIT_TYPE_AFTERTHROW)) {
				String exceptionStr =  map.get("exceptionStr").toString();
				actMessage = new StringBuilder("AuditAfterThrow审计,").append(actMessage).append(",exception:").append(exceptionStr).toString();
			}
			logger.info(actMessage);
			
			auditLog.setActMessage(actMessage);
			this.insert(auditLog);
    	}
	}	
	
	/**
	* @Description: 根据警告配置动态加载相应的警告通知类及调用类中的方法。
	* 				执行警告通知的方法参数应该为：alertContent,alertTarget
	* @author: gzx
	* @date: 2019年10月28日 下午6:34:30 
	* @param auditLevel  行为审核级别实体
	* @param userId		  用户ID
	* @param userName    用户名称
	* @param className	 切面类名称
	* @param method		 切面方法	
	* @param actTime	 行为时间
	* @param @return 参数 
	* @return String 返回类型   0：失败。1：成功
	* @throws
	 */
    public String doAlarm(AuditLevel auditLevel,String userId,String userName,String className,String method,String actTime) {
    	//审计级别如果是-1，无需发送警告通知 。如果需要发送警告通知，警告对象也不能为空
    	if(AuditConstants.AUDIT_LEVEL_IGNORE != auditLevel.getLevelId()&& auditLevel.getAlertTarget() != null) {
	    	String alarmClassName = auditLevel.getAuditAlert().getAlertClass(); 
			String alarmMethod = auditLevel.getAuditAlert().getClassMethod();
			//动态调用类及方法
			Class<?> alarmClass = SpringContextHolder.getBean(StringUtils.toLowerString(alarmClassName)).getClass();
			
			//警告内容
			String alertContent = new StringBuilder("警告等级:").append(auditLevel.getLevelName()).append("。").append(userName)
					.append("在").append(actTime).append("操作了类").append(className).append("中的方法：")
					.append(method).append("。userId是：").append(userId).toString();
			try {
				alarmClass.getMethod(alarmMethod, new Class[]{String.class,String.class})
						.invoke(SpringContextHolder.getBean(StringUtils.toLowerString(alarmClassName)), 
								new Object[]{alertContent,auditLevel.getAlertTarget()});
				return  "1";
			} catch (IllegalAccessException e) {
				logger.error("审计警告调用警告方法异常，异常信息为：{},警告内容：{}",e.getMessage(),alertContent);
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				logger.error("审计警告调用警告方法异常，异常信息为：{},警告内容：{}",e.getMessage(),alertContent);
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				logger.error("审计警告调用警告方法异常，异常信息为：{},警告内容：{}",e.getMessage(),alertContent);
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				logger.error("审计警告调用警告方法异常，异常信息为：{},警告内容：{}",e.getMessage(),alertContent);
				e.printStackTrace();
			} catch (SecurityException e) {
				logger.error("审计警告调用警告方法异常，异常信息为：{},警告内容：{}",e.getMessage(),alertContent);
				e.printStackTrace();
			}
			return "0";
    	}
    	return "1";
	}

}