package com.loit.v1.modules.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.ResourceTypeDTO;
import com.loit.v1.modules.entity.ResourceType;
import com.loit.v1.modules.service.ResourceTypeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 资源类别管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/resourceType")
@Api(value = "ResourceTypeController", tags = {"应用资源类别管理Controller"})
public class ResourceTypeController extends BaseController {

	@Autowired
	private ResourceTypeService resourceTypeService;
	
	/**
	 * @description 资源类别列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param applicationManage 资源类别管理实体类	
	 * @return
	 */
	@GetMapping(value = "list")
	@ApiOperation(value="列表", notes="获取 资源类别列表")	
	public AjaxJson list(@ApiIgnore ResourceType resourceType) {
		List<ResourceType> list = resourceTypeService.findAllList(resourceType);
		//logger.debug("获取资源类别列表成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}


	/**
	 * @Description 资源类别详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param typeId
	 * @return
	 */
	@GetMapping(value = "{typeId}")
	@ApiOperation(value="详情", notes="根据typeId获取资源类别详情")
	@ApiImplicitParam(name = "typeId", value = "资源类别ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String typeId) {	
		if(StringUtils.isBlank(typeId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ResourceType resourceType  = resourceTypeService.get(typeId);
		//logger.debug("获取资源类别详情成功");
		AjaxJson json = new AjaxJson();
		json.put("obj", resourceType);
		return json;
	}


	/**
	 * @Description 新增资源类别
	 * @author xutao
	 * @date：2019年10月18日
	 * @param resourceTypeDTO
	 * @return
	 */
	@PostMapping(value = "save")
	@ApiOperation(value = "保存", notes = "新增资源类别")
	@ApiImplicitParam(name = "resourceTypeDTO", value = "资源类别数据传输实体类", required = true, dataType = "ResourceTypeDTO")
	public AjaxJson save(@RequestBody ResourceTypeDTO resourceTypeDTO) {
		AjaxJson json = new AjaxJson();	
		ResourceType resourceType = ModelMapperUtils.map(resourceTypeDTO, ResourceType.class);
		try {
			List<ResourceType> list = resourceTypeService.findList(resourceType);
			if(list.size()>0) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.RESOURCETYPE_IS_EXIST);
			}else {
				resourceTypeService.insert(resourceType);
				json.setMsg("保存成功");
			}			
			return json;
		} catch (Exception e) {
			logger.error("info:资源类别保存失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}		
	}
	
	
	/**
	 * @Description 修改资源类别
	 * @param typeId
	 * @param resourceTypeDTO
	 * @return
	 */
	@PostMapping(value = "{typeId}")
	@ApiOperation(value = "修改", notes = "修改资源类别")
	public AjaxJson update(@PathVariable int typeId,@RequestBody ResourceTypeDTO resourceTypeDTO) {
		AjaxJson json = new AjaxJson();
		ResourceType resourceType = ModelMapperUtils.map(resourceTypeDTO, ResourceType.class);
		try {
			resourceType.setTypeId(typeId);
			resourceTypeService.update(resourceType);
			json.setMsg("修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:应用资源类别修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);	
		}		
	}

	/**
	 * @Description 资源类别删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除应用资源，多ID时，以逗号分隔")
	@DeleteMapping(value = "{typeId}")
	public AjaxJson deleteAll(@PathVariable("typeId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			ResourceType resourceType = new ResourceType();
			resourceTypeService.deleteAll(ids,resourceType);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);	
		}
	}
	
	/**
	 * @description 检查资源类别名称是否重复
	 * @param typeId
	 * @param typeName
	 * @return
	 */
	@ApiOperation(value = "检查资源类别名称是否重复", notes = "检查资源类别名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "typeId", value = "资源类别id", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "typeName", value = "资源类别名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkTypeNameUnique")
	public AjaxJson checkTypeNameUnique(Integer typeId,String typeName) {
		boolean uniqueFlag = resourceTypeService.checkNameUnique(typeId, typeName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("资源类别名称不重复");
		} else {
			json.setMsg("资源类别名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
}
