package com.loit.v1.modules.handler;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.loit.common.utils.RedisUtil;
import com.loit.common.utils.StringUtils;
import com.loit.loitcasclient.utils.ClientSpringContextUtil;
import com.loit.v1.modules.constant.BusinessConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.TaskExecutor;
import org.springframework.web.socket.*;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * @description: 处理消息的接收和发送
 * @date: 2020/4/26 15:26
 * @author: wubj
 * @version v1.0
 */
@Configuration
public class MyWebSocketHandler implements WebSocketHandler {

    private static Logger LOGGER = LoggerFactory.getLogger(MyWebSocketHandler.class);

    private static ReentrantReadWriteLock lock = new ReentrantReadWriteLock();

    // 保存用户链接，Map来存储WebSocketSession，key用USER_ID 即在线用户列表
    private static final Set<WebSocketSession> users = new HashSet<>();

    // 用户标识，对应监听器的key
    private static final String USER_ID = "userId";

    private TaskExecutor taskExecutor;

    // 建立连接后触发的回调
    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        LOGGER.debug("====================WebSocket连接成功====================");
        lock.writeLock().lock();
        try {
            users.add(session);
            this.updateServerInfo();
        } catch (Exception e) {
            LOGGER.error("====================用户登录失败====================");
        } finally {
            lock.writeLock().unlock();
        }
        LOGGER.info("用户：" + session.getAttributes().get(USER_ID) + "已上线！ 当前在线用户数量：" + users.size());
    }

    // 将服务器信息存到redis
    private void updateServerInfo() {
        JSONObject jsonObject = new JSONObject();
        Set<String> set = new HashSet<>();
        for (WebSocketSession user : users) {
            set.add((String) user.getAttributes().get(USER_ID));
        }
        String userIds = String.join(",", set);
        String address = HandshakeInterceptor.address;
        jsonObject.put("userIds", userIds);
        jsonObject.put("address", address);

        String serverInfo = (String) RedisUtil.get(BusinessConstants.WEBSOCKET_CATCH_KEY);
        JSONArray jsonArray = new JSONArray();
        Boolean flag = false;
        if (StringUtils.isNotEmpty(serverInfo)) {
            jsonArray = JSON.parseArray(serverInfo);
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject object = (JSONObject) jsonArray.get(i);
                String server = object.getString("address");
                if (address.equals(server)) {
                    object.put("userIds", userIds);
                    flag = true;
                    break;
                }
            }
            if (!flag) {
                jsonArray.add(jsonObject);
            }
        } else {
            jsonArray.add(jsonObject);
        }
        RedisUtil.set(BusinessConstants.WEBSOCKET_CATCH_KEY, jsonArray.toString());
    }

    // 传输消息出错时触发的回调
    @Override
    public void handleTransportError(WebSocketSession session, Throwable throwable) throws Exception {
        LOGGER.error("====================handleTransportError====================");
        session.close();
        users.remove(session);
        this.updateServerInfo();
    }

    // 断开连接后触发的回调
    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        LOGGER.info("用户：" + session.getAttributes().get(USER_ID) + "已离线，原因：" + closeStatus);
        session.close();
        users.remove(session);
        this.updateServerInfo();
    }

    // 是否支持分包
    @Override
    public boolean supportsPartialMessages() {
        return false;
    }

    // 收到消息时触发的回调
    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> webSocketMessage) {
        LOGGER.info("====================handleMessage====================");
        String message = (String) webSocketMessage.getPayload();
        JSONArray jsonArray = JSON.parseArray(message);
        taskExecutor = (TaskExecutor) ClientSpringContextUtil.getBean(TaskExecutor.class);
        long startTime=System.currentTimeMillis();
        ExecutorCompletionService<String> completionService = new ExecutorCompletionService<String>(taskExecutor);
        for (Object object : jsonArray) {
            completionService.submit(() -> {
                JSONObject jsonObject = (JSONObject) object;
                String userId = jsonObject.getString("userId");
                String msg = jsonObject.getString("msg");
                this.sendMessageToUsers(userId, msg);
                return userId;
            });
        }
        long endTime=System.currentTimeMillis();
        float excTime=(float)(endTime-startTime)/1000;
        LOGGER.info("====================WebSocket推送消息，用时：" + excTime + "秒====================");
    }

    // 广播所有在线用户
    public static void sendMessageBroadcast(String message) {
        try {
            for (WebSocketSession user : users) {
                if (user.isOpen()) {
                    user.sendMessage(new TextMessage(message));
                }
            }
        } catch (IOException e) {
            LOGGER.error("====================WebSocket消息发送失败====================");
        }
    }

    // 发送指定用户
    public static void sendMessageToUsers(String receiveId, String message) {
        try {
            for (WebSocketSession user : users) {
                String userId = (String) user.getAttributes().get(USER_ID);
                if (user.isOpen() && receiveId.equals(userId)) {
                    user.sendMessage(new TextMessage(message));
                }
            }
        } catch (IOException e) {
            LOGGER.error("====================WebSocket消息发送失败====================");
        }
    }

}

