package com.loit.v1.modules.service;

import com.loit.api.dto.Message;
import com.loit.api.service.MessageApiService;
import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.DictData;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.DictDataUtils;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.NoticeDao;
import com.loit.v1.modules.entity.Notice;
import com.loit.v1.modules.entity.NoticeReceive;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;


/**
 * 通知表服务实现.
 * Created by WBJ on 2020-04-23.
 **/
@Service
@Transactional(readOnly = true)
public class NoticeService extends CrudService<NoticeDao, Notice> {

    @Autowired
    private NoticeReceiveService noticeReceiveService;

    @Autowired
    private PersonBaseService personBaseService;

    @Autowired
    private MessageApiService messageApiService;

    // 批量存储数据个数
    private static final int NOTICE_NUM = 5000;

    // 字典map
    private Map<String, String> dictMap = new HashMap<>();

    /**
     * 通知管理分页列表
     * @param page 分页对象
     * @param notice
     * @return
     */
    public Page<Notice> findPage(Page<Notice> page, Notice notice) {
        notice.setPage(page);
        // 当前登录用户名
        User user = UserUtils.getUser();
        notice.setSendId(user.getUserId());
        List<Notice> list = dao.findList(notice);
        if (ListUtil.isNotEmpty(list)) {
            this.getDictMap();

            for (Notice entity : list) {
                if (StringUtils.isNotEmpty(entity.getNoticeType()) && StringUtils.isNotEmpty(dictMap.get(entity.getNoticeType()))) {
                    entity.setNoticeTypeName(dictMap.get(entity.getNoticeType()));
                }
            }
        }
        page.setList(list);
        return page;
    }

    /**
     * 通知详情
     * @param id
     * @return
     */
    public Notice get(Integer id) {
        Notice notice = dao.get(id);
        if (notice != null) {
            this.getDictMap();
            notice.setNoticeTypeName(dictMap.get(notice.getNoticeType()));
        }
        // 通知接收人信息
        NoticeReceive receive = new NoticeReceive();
        receive.setNoticeId(id);
        List<NoticeReceive> receiveList = noticeReceiveService.findList(receive);
        // 获取姓名和部门信息
        if (ListUtil.isNotEmpty(receiveList)) {
            List<String> receiveIds = receiveList.stream().map(p->p.getReceiveId()).collect(Collectors.toList());
            List<PersonBaseDTO> personBases = personBaseService.getAccountByUserIds(receiveIds);
            if (ListUtil.isNotEmpty(personBases)) {
                Map<String,PersonBaseDTO> map = personBases.stream().collect(Collectors.toMap(PersonBaseDTO::getUserId, i -> i));
                for (NoticeReceive noticeReceive : receiveList) {
                    if (noticeReceive.getReceiveId() != null && map != null && map.get(noticeReceive.getReceiveId()) != null) {
                        noticeReceive.setReceiveName(map.get(noticeReceive.getReceiveId()).getPersonName());
                        noticeReceive.setOrgName(map.get(noticeReceive.getReceiveId()).getOrgName());
                    }
                }
            }
            notice.setReceiveList(receiveList);
        }
        return notice;
    }

    /**
     * 推送通知
     * @param notice
     */
    @Transactional(readOnly = false)
    public void publish(Notice notice) {
        // 保存通知实体
        this.save(notice);
        // 发布通知
        this.publishStatus(notice);
    }

    /**
     * 批量插入数据到通知接收表
     * @param receiveList
     */
    private void insertNoticeReceiveList(List<NoticeReceive> receiveList){
        int listSize = receiveList.size();
        int toIndex = NOTICE_NUM;
        for (int i = 0; i < listSize; i += NOTICE_NUM) {
            // 作用为toIndex最后没有1000条数据则剩余几条newList中就装几条
            if (i + NOTICE_NUM > listSize) {
                toIndex = listSize - i;
            }
            List<NoticeReceive> newList = receiveList.subList(i, i + toIndex);
            // 批量插入
            noticeReceiveService.insertBatch(newList);
        }
    }

    /**
     * 新增或修改通知
     * @param notice
     */
    @Transactional(readOnly = false)
    public void save(Notice notice) {
        if (notice.getId() != null) {
            // 修改通知实体
            this.update(notice);
            // 修改接收人信息
            noticeReceiveService.deleteByNoticeIds(Arrays.asList(notice.getId().longValue()));
        } else {
            // 保存通知实体
            User user = UserUtils.getUser();
            notice.setSendId(user.getUserId());
            notice.setSendName(user.getPersonName());
            notice.setPublishStatus("0");
            this.insert(notice);
        }

        // 判断是否保存通知接收表
        int noticeId = notice.getId();
        List<String> accoutList = new ArrayList<>();
        List<NoticeReceive> receiveList = new ArrayList<>();

        if (StringUtils.isNotEmpty(notice.getReceiveIds())) {
            // 存储通知接收人信息
            String[] receiveIds = notice.getReceiveIds().split(",");
            accoutList = Arrays.asList(receiveIds);
        }

        if (ListUtil.isNotEmpty(accoutList)) {
            for (String receiveId : accoutList) {
                NoticeReceive noticeReceive = new NoticeReceive();
                noticeReceive.setNoticeId(noticeId);
                noticeReceive.setReceiveId(receiveId);
                noticeReceive.setNoticeStatus("0");
                noticeReceive.setCreateTime(new Date());
                receiveList.add(noticeReceive);
            }
            // 批量存储通知接收表信息
            this.insertNoticeReceiveList(receiveList);
        }
    }

    /**
     * 通知修改为已发布状态
     * @param notice
     */
    @Transactional(readOnly = false)
    public void publishStatus(Notice notice) {
        // 通知修改为已发布状态
        List<Long> ids = new ArrayList<>();
        Integer noticeId = notice.getId();
        ids.add(noticeId.longValue());
        dao.updatePublishStatus(ids);

        // 获取接收人信息
        List<String> receiveIds = new ArrayList<>();
        NoticeReceive receive = new NoticeReceive();
        receive.setNoticeId(noticeId);
        List<NoticeReceive> receiveList = noticeReceiveService.findList(receive);
        if (ListUtil.isNotEmpty(receiveList)) {
            receiveIds = receiveList.stream().map(p->p.getReceiveId()).collect(Collectors.toList());
        }
        notice = dao.get(noticeId);

        try {
            // websocket推送
            Message message = new Message();
            message.setNoticeTopic(notice.getNoticeTopic());
            message.setNoticeContent(notice.getNoticeContent());
            message.setNoticeType(notice.getNoticeType());
            message.setNoticeSource("NOTICE_SOURCE_PORTAL");
            message.setReceiveIds(StringUtils.join(receiveIds, ","));
            message.setAllFlag("0");
            message.setNoticeLevel("1");
            message.setNoticeTypeId(noticeId);
            messageApiService.propel(message);
        } catch (Exception e) {
            logger.error("消息推送异常：{}", e.getMessage());
        }
    }

    /**
     * 通知删除
     * @param ids
     */
    @Transactional(readOnly = false)
    public void deleteAll(String ids) {
        if (StringUtils.isNotEmpty(ids)) {
            List<Long> noticeIds = new ArrayList<>();
            String[] str = ids.split(",");
            for (String id : str) {
                noticeIds.add(Long.valueOf(id));
            }
            noticeReceiveService.deleteByNoticeIds(noticeIds);
            dao.deleteByIds(noticeIds);
        }
    }

    /**
     * 获取字典数据
     */
    private void getDictMap() {
        if (dictMap == null || dictMap.size() == 0) {
            // 获取字典信息
            List<DictData> typeDict = DictDataUtils.getDictDataList(BusinessConstants.NOTICE_TYPE);
            for (DictData dictData : typeDict) {
                dictMap.put(dictData.getDataCode(), dictData.getDataName());
            }
        }
    }
}