package com.loit.v1.modules.service;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.loit.common.json.AjaxJson;
import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.api.dto.RoleDTO;
import com.loit.v1.api.dto.SynLogDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.ApporgShortcut;
import com.loit.v1.modules.entity.SynLog;
import com.loit.v1.modules.utils.Model.HttpUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.JmsException;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;
import com.loit.common.service.CrudService;
import com.loit.v1.modules.dao.SynTargetDao;
import com.loit.v1.modules.entity.SynTarget;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;


/**
 * @author zhangyl
 * @version V1.0
 * @ClassName: SynTargetService
 * @Description: 同步目标
 * @date 2019-11-20 17:29:52
 **/
@Service
public class SynTargetService extends CrudService<SynTargetDao, SynTarget> {

    @Autowired
    private RoleService roleService;
    @Autowired
    private OrgService orgService;
    @Autowired
    private PersonBaseService personBaseService;
    @Autowired
    private SynLogService synLogService;

    /**
     * 组织同步（增加、修改） 消息消费者
     *
     * @param message
     * @return void
     */
    @JmsListener(destination = BusinessConstants.QUEUE_SYNC_ORG_SAVE)
    @Transactional(readOnly = false)
    public void orgSyncSaveConsumer(MessageModel message) throws JmsException {
        if (message == null) {
            return;
        }
        Map<String, Object> syncMap = new HashMap<>();
        SynLogDTO synLog = new SynLogDTO();
        ObjectMapper objectMapper = new ObjectMapper();
        OrgDTO org = objectMapper.convertValue(message.getMap().get("org"), OrgDTO.class);
        boolean isInsert = (boolean) message.getMap().get("isInsert");
        //判断是新增还是修改，true是新增,false是修改
        if (isInsert) {
            syncMap.put("synMode", "1");//同步方式为新增
        } else {
            syncMap.put("synMode", "2");//同步方式为修改
        }

        List<SynTarget> synTargetList = dao.findListByOrgId(org.getOrgId(), syncMap.get("synMode").toString());
        //遍历所有含该组织的同步目标
        for (SynTarget synTarget : synTargetList) {
            //判断该应用是否被禁用
            if (synTarget.getEnabled().equals("2")) {
                continue;
            }
            syncMap.put("org", org);
            //日志处理
            synLog.setObjectId(org.getOrgId());
            synLog.setObjectName(org.getOrgName());
            synLog.setTargetId(synTarget.getTargetId());
            synLog.setCreateTime(new Date());
            //发送同步数据给第三方应用
            String ajaxJsonString = JSONObject.toJSONString(syncMap);
            boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
            if (!flag) {
                synLog.setSynStatus("2"); //同步失败
                synLogService.insertLog(synLog);//插入日志
                continue;
            }
            synLog.setSynStatus("1");//成功
            synLogService.insertLog(synLog);//插入日志
        }
    }

    /**
     * 组织同步（删除） 消息消费者
     *
     * @param message
     * @return void
     */
    @JmsListener(destination = BusinessConstants.QUEUE_SYNC_ORG_DELETE)
    @Transactional(readOnly = false)
    public void orgSyncDeleteConsumer(MessageModel message) throws JmsException {
        if (message == null) {
            return;
        }
        Map<String, Object> syncMap = new HashMap<>();
        syncMap.put("synMode", "3");//同步方式为删除
        //取出消息
        List<ApporgShortcut> apporgShortcutList = (List<ApporgShortcut>) message.getMap().get("apporgShortcutList");
        List<OrgDTO> orgList = (List<OrgDTO>) message.getMap().get("orgList");

        for (int i = 0; i < apporgShortcutList.size(); i++) {
            ApporgShortcut apporgShortcut = apporgShortcutList.get(i);
            SynTarget synTarget = dao.getByAppIdFromOrg(apporgShortcut.getAppId(), syncMap.get("synMode").toString());
            if (synTarget != null) {
                //遍历同步目标 给符合要求的目标同步数据
                syncMap.put("orgId", apporgShortcut.getOrgId());
                //判断该应用是否被禁用
                if (synTarget.getEnabled().equals("2")) {
                    continue;
                }
                //日志维护
                SynLogDTO synLog = new SynLogDTO();
                synLog.setObjectId(apporgShortcut.getOrgId());
                //匹配org表与apporgShortcut表OrgId,相同则是同一条数据
                for (OrgDTO orgDTO : orgList) {
                    if (orgDTO.getOrgId().equals(apporgShortcut.getOrgId())) {
                        synLog.setObjectName(orgDTO.getOrgName());
                    }
                }
                synLog.setTargetId(synTarget.getTargetId());
                synLog.setCreateTime(new Date());
                //发送同步数据给第三方应用
                boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), JSONObject.toJSONString(syncMap));
                if (!flag) {
                    synLog.setSynStatus("2"); //同步失败
                    synLogService.insertLog(synLog);//插入日志
                    continue;
                }
                synLog.setSynStatus("1");//成功
                synLogService.insertLog(synLog);//插入日志
            }
        }
    }

    /**
     * 人员同步（增加、修改） 消息消费者
     *
     * @param message
     * @return void
     */
    @JmsListener(destination = BusinessConstants.QUEUE_SYNC_PERSON_SAVE)
    @Transactional(readOnly = false)
    public void personSyncSaveConsumer(MessageModel message) throws JmsException {
        if (message == null) {
            return;
        }
        Map<String, Object> syncMap = new HashMap<>();
        SynLogDTO synLog = new SynLogDTO();
        ObjectMapper objectMapper = new ObjectMapper();
        PersonBaseDTO person = objectMapper.convertValue(message.getMap().get("person"), PersonBaseDTO.class);
        boolean isInsert = (boolean) message.getMap().get("isInsert");
        //判断是新增还是修改，true是新增,false是修改
        if (isInsert) {
            syncMap.put("synMode", "1");//同步方式为新增
        } else {
            syncMap.put("synMode", "2");//同步方式为修改
        }
        List<SynTarget> synTargetList = dao.findListByAccountId(person.getAccountId(), syncMap.get("synMode").toString());
        //中间表出现数据错误会造成查不到synTargetList，为空则不作同步操作
        if (synTargetList == null) {
            return;
        }
        //遍历所有含该组织的同步目标
        for (SynTarget synTarget : synTargetList) {
            syncMap.put("person", person);
            //判断该应用是否被禁用
            if (synTarget.getEnabled().equals("2")) {
                continue;
            }
            //日志维护
            synLog.setObjectId(person.getAccountId());
            synLog.setObjectName(person.getPersonName());
            synLog.setTargetId(synTarget.getTargetId());
            synLog.setCreateTime(new Date());
            //发送同步数据给第三方应用
            String ajaxJsonString = JSONObject.toJSONString(syncMap);
            boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
            if (!flag) {
                synLog.setSynStatus("2"); //同步失败
                synLogService.insertLog(synLog);//插入日志
                continue;
            }
            synLog.setSynStatus("1");//成功
            synLogService.insertLog(synLog);//插入日志
        }
    }

    /**
     * 人员同步（删除） 消息消费者
     *
     * @param message
     * @return void
     */
    @JmsListener(destination = BusinessConstants.QUEUE_SYNC_PERSON_DELETE)
    @Transactional(readOnly = false)
    public void personSyncDeleteConsumer(MessageModel message) throws JmsException {
        if (message == null) {
            return;
        }
        Map<String, Object> syncMap = new HashMap<>();
        syncMap.put("synMode", "3");//同步方式为删除
        //取出消息
        List<ApporgShortcut> apporgShortcutList = (List<ApporgShortcut>) message.getMap().get("apporgShortcutList");
        List<PersonBaseDTO> accountList = (List<PersonBaseDTO>) message.getMap().get("accountList");

        for (int i = 0; i < apporgShortcutList.size(); i++) {
            ApporgShortcut apporgShortcut = apporgShortcutList.get(i);
            SynTarget synTarget = dao.getByAppIdFromAccount(apporgShortcut.getAppId(), syncMap.get("synMode").toString());
            if (synTarget != null) {
                //遍历同步目标 给符合要求的目标同步数据
                syncMap.put("accountId", apporgShortcut.getAccountId());
                //判断该应用是否被禁用
                if (synTarget.getEnabled().equals("2")) {
                    continue;
                }
                //日志维护
                SynLogDTO synLog = new SynLogDTO();
                synLog.setObjectId(apporgShortcut.getAccountId());
                //匹配账号表accountId与应用组织快捷表accountId，相等则为同一条数据
                for (PersonBaseDTO personBaseDTO : accountList) {
                    if (personBaseDTO.getAccountId().equals(apporgShortcut.getAccountId()))
                        synLog.setObjectName(personBaseDTO.getPersonName());
                }
                synLog.setTargetId(synTarget.getTargetId());
                synLog.setCreateTime(new Date());
                //发送同步数据给第三方应用
                boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), JSONObject.toJSONString(syncMap));
                if (!flag) {
                    synLog.setSynStatus("2"); //同步失败
                    synLogService.insertLog(synLog);//插入日志
                    continue;
                }
                synLog.setSynStatus("1");//成功
                synLogService.insertLog(synLog);//插入日志
            }
        }
    }

    /**
     * 角色同步（新增、修改） 消息消费者
     *
     * @param message
     * @return void
     */
    @JmsListener(destination = BusinessConstants.QUEUE_SYNC_ROLE_SAVE)
    @Transactional(readOnly = false)
    public void roleSyncSaveConsumer(MessageModel message) throws JmsException {
        if (message == null) {
            return;
        }
        Map<String, Object> syncMap = new HashMap<>();
        SynLogDTO synLog = new SynLogDTO();
        //接受消息并转换成对应对象
        ObjectMapper objectMapper = new ObjectMapper();
        RoleDTO role = objectMapper.convertValue(message.getMap().get("role"), RoleDTO.class);
        boolean isInsert = (boolean) message.getMap().get("isInsert");
        if (isInsert) {
            syncMap.put("synMode", "1");//同步方式为新增
        } else {
            syncMap.put("synMode", "2");//同步方式为修改
        }
        SynTarget synTarget = dao.getByRoleId(role.getAppId(), syncMap.get("synMode").toString());
        if (synTarget != null) {
            syncMap.put("role", role);
            //判断该应用是否被禁用
            if (synTarget.getEnabled().equals("2")) {
                return;
            }
            //日志维护

            synLog.setObjectId(role.getRoleId());
            synLog.setObjectName(role.getRoleName());
            synLog.setTargetId(synTarget.getTargetId());
            synLog.setCreateTime(new Date());
            //发送同步数据给第三方应用
            String ajaxJsonString = JSONObject.toJSONString(syncMap);
            boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
            if (!flag) {
                synLog.setSynStatus("2"); //同步失败
                synLogService.insertLog(synLog);//插入日志
                return;
            }
            synLog.setSynStatus("1");//成功
            synLogService.insertLog(synLog);//插入日志
        }
    }

    /**
     * 角色同步（删除） 消息消费者
     *
     * @param message
     * @return void
     */
    @JmsListener(destination = BusinessConstants.QUEUE_SYNC_ROLE_DELETE)
    @Transactional(readOnly = false)
    public void roleSyncDeleteConsumer(MessageModel message) throws JmsException {
        if (message == null) {
            return;
        }
        //取出消息
        List<RoleDTO> roleList = (List<RoleDTO>) message.getMap().get("roleList");
        Map<String, Object> syncMap = new HashMap<>();
        syncMap.put("synMode", "3");//同步方式为删除
        for (int i = 0; i < roleList.size(); i++) {
            RoleDTO role = new RoleDTO();
            BeanUtils.copyProperties(roleList.get(i), role);
            SynTarget synTarget = dao.getByRoleId(role.getAppId(), syncMap.get("synMode").toString());
            if (synTarget != null) {
                //判断该应用是否被禁用
                if (synTarget.getEnabled().equals("2")) {
                    continue;
                }
                syncMap.put("roleId", role.getRoleId());//同步方式为删除
                //日志维护
                SynLogDTO synLog = new SynLogDTO();
                synLog.setObjectId(role.getRoleId());
                synLog.setObjectName(role.getRoleName());
                synLog.setTargetId(synTarget.getTargetId());
                synLog.setCreateTime(new Date());
                //发送同步数据给第三方应用
                String ajaxJsonString = JSONObject.toJSONString(syncMap);
                boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
                if (!flag) {
                    synLog.setSynStatus("2"); //同步失败
                    synLogService.insertLog(synLog);//插入日志
                    continue;
                }
                synLog.setSynStatus("1");//成功
                synLogService.insertLog(synLog);//插入日志
            }
        }
    }

    /**
     * 全量同步
     *
     * @param targetId
     * @return AjaxJson
     */
    @Transactional(readOnly = false)
    public AjaxJson allSync(String targetId) {

        AjaxJson ajaxJson = new AjaxJson();
        Map<String, Object> syncMap = new HashMap<>();
        SynTarget synTarget = get(targetId);
        int appId = synTarget.getAppId();
        //判断是否被禁用
        if (synTarget.getEnabled().equals("2")) {
            return AjaxJson.returnExceptionInfo("当前数据同步已被【禁用】");
        }
        //当前操作非增加，不允许全量同步
        if (!synTarget.getSynMode().equals("1")) {
            return AjaxJson.returnExceptionInfo("当前操作非增加，不允许全量同步");
        }
        //判断是否已经对该目标进行过全量同步操作
        List<SynLog> list = synLogService.findListByTargetId(Integer.valueOf(targetId));
        if (list.size() > 0) {
            return AjaxJson.returnExceptionInfo("已经对该目标进行了全量同步，无需重复操作");
        }
        syncMap.put("synMode", synTarget.getSynMode());//发送同步方式
        //同步组织
        if (synTarget.getObjectType().equals("1")) {
            List<OrgDTO> rogList = orgService.findListByAppId(appId);
            for (OrgDTO org : rogList) {
                syncMap.put("org", org);
                //同步日志记录
                SynLog synLog = new SynLog();
                synLog.setObjectId(org.getOrgId());
                synLog.setObjectName(org.getOrgName());
                synLog.setTargetId(Integer.valueOf(targetId));
                synLog.setCreateTime(new Date());
                String ajaxJsonString = JSONObject.toJSONString(syncMap);
                boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
                if (!flag) {
                    synLog.setSynStatus("2"); //同步失败
                    synLogService.insert(synLog);//插入日志
                    continue;
                }
                synLog.setSynStatus("1");//成功
                synLogService.insert(synLog);//插入日志
            }
        }
        // 同步人员
        else if (synTarget.getObjectType().equals("2")) {
            List<PersonBaseDTO> personBaseDTOList = personBaseService.findListByAppId(appId);
            for (PersonBaseDTO personBase : personBaseDTOList) {
                syncMap.put("person", personBase);
                //同步日志记录
                SynLog synLog = new SynLog();
                synLog.setObjectId(personBase.getAccountId());
                synLog.setObjectName(personBase.getPersonName());
                synLog.setTargetId(Integer.valueOf(targetId));
                synLog.setCreateTime(new Date());
                String ajaxJsonString = JSONObject.toJSONString(syncMap);
                boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
                if (!flag) {
                    synLog.setSynStatus("2"); //同步失败
                    synLogService.insert(synLog);//插入日志
                    continue;
                }
                synLog.setSynStatus("1");//成功
                synLogService.insert(synLog);//插入日志
                System.out.println("\033[34;0m" + "[synLog]" + synLog + "\033[0m");
            }
            //同步角色
        } else if (synTarget.getObjectType().equals("3")) {
            List<RoleDTO> roleList = roleService.findListByAppId(appId);
            for (RoleDTO role : roleList) {
                syncMap.put("role", role);
                //同步日志记录
                SynLog synLog = new SynLog();
                synLog.setObjectId(role.getRoleId());
                synLog.setObjectName(role.getRoleName());
                synLog.setTargetId(Integer.valueOf(targetId));
                synLog.setCreateTime(new Date());
                String ajaxJsonString = JSONObject.toJSONString(syncMap);
                boolean flag = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), ajaxJsonString);
                if (!flag) {
                    synLog.setSynStatus("2"); //同步失败
                    synLogService.insert(synLog);//插入日志
                    continue;
                }
                synLog.setSynStatus("1");//成功
                synLogService.insert(synLog);//插入日志
            }
        }
        return ajaxJson;
    }
}