package com.loit.v1.oauth.service;

import com.loit.api.service.OnlineUserCountApiService;
import com.loit.common.utils.DateUtils;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.util.DateTotalUtils.DateTotalUtils;
import com.loit.v1.modules.entity.AccountPolicy;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.service.AccountPolicyService;
import com.loit.v1.modules.service.AccoutInfoService;
import com.loit.v1.modules.service.IpPolicyAccountService;
import com.loit.v1.modules.utils.Model.IpUtil;
import com.loit.v1.modules.utils.Model.PasswordCheckUtils;
import com.loit.v1.oauth.data.CasStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 统一登录验证服务
 */
@Service
public class LoginOauthValidateService {

    @Autowired
    private AccoutInfoService accoutInfoService;
    @Autowired
    private IpPolicyAccountService ipPolicyAccountService;
    @Autowired
    private AccountPolicyService accountPolicyService;
    @Autowired
    private OnlineUserCountApiService onlineUserCountApiService;
    @Autowired
    private CaptchaService captchaService;
    @Autowired
    private AsyncTaskService asyncTaskService;

    /**
     * 密码错误次数缓存, 正确后清除
     */
    private static Map<String, Integer> passwordErrorCountCache = new HashMap<>();

    /**
     * 验证账号
     * @param userName 用户名
     * @return
     */
    public CasStatus loginValidate(String userName){
        Boolean isAdmin = "admin".equals(userName) ? true : false;
        // 根据用户名获取用户信息
        AccoutInfo user = accoutInfoService.getAccountByUserId(userName);
        // 验证账号
        CasStatus status = this.validate(user, isAdmin);
        return status;
    }

    /**
     * 验证账号和密码
     * @param userName 用户名
     * @param password 密码
     * @return
     */
    public CasStatus loginValidate(String userName, String password){
        Boolean isAdmin = "admin".equals(userName) ? true : false;
        // 根据用户名获取用户信息
        AccoutInfo user = DateTotalUtils.printR(
                "1.2.根据用户名获取用户信息accoutInfoService.getAccountByUserId(userName)",
                AccoutInfo.class,
                () -> accoutInfoService.getAccountByUserId(userName)
        );
        // 验证账号
        CasStatus status = DateTotalUtils.printR(
                "1.3.验证账号validate(user, isAdmin)",
                CasStatus.class,
                ()->this.validate(user, isAdmin)
        );
        if(null == status){
            // 验证密码
            status =  DateTotalUtils.printR(
                    "1.4.验证密码 passwordValidate(password, isAdmin, user)",
                    CasStatus.class,
                    ()->this.passwordValidate(password, isAdmin, user)
            );
        }
        return status;
    }

    /**
     * 验证账号和密码和验证码
     * @param userName 用户名
     * @param password 密码
     * @return
     */
    public CasStatus loginValidate(String userName, String password, boolean captchaSwitch, String captcha){
        if(captchaSwitch){
            if(StringUtils.isEmpty(captcha)){
                return CasStatus.STATUS_911;
            }
        }
        CasStatus status = loginValidate(userName, password);
        if(null == status) {
            status = captchaValidate(captchaSwitch, captcha);
        }
        return status;
    }

    /**
     * 私有方法-账号登录验证
     * 1. cas 服务端会通过post请求，并且把用户信息以"用户名:密码"进行Base64编码放在authorization请求头中
     * 2. 返回200状态码并且格式为{"@class":"org.apereo.cas.authentication.principal.SimplePrincipal","id":"casuser","attributes":{}} 是成功的
     *
     * @param isAdmin 是超管用户, 放过一些验证规则
     */
    private CasStatus validate(AccoutInfo user, Boolean isAdmin){
        // 其他用户(不是超管用户)
        Boolean isNotAdmin = !isAdmin;

        // 账号不存在
        if (user == null || user.getDelFlag().equals("1")) {
            return CasStatus.STATUS_902;
        }

        // 账号已被锁定: 账号状态（0正常,1锁定,2一般管理员禁止）
        if(isNotAdmin && user.getUserStat().equals("1")){
            return CasStatus.STATUS_903;
        }

        // 账号被管理员禁止
        if(isNotAdmin && user.getUserStat().equals("2") ){
            return CasStatus.STATUS_904;
        }

        // IP访问策略限制: 设置客户端最后登录的ip
        user.setLastLoginIp(IpUtil.clientIp());
        Boolean aBoolean = DateTotalUtils.printR(
                "1.3.1.IP访问策略限制: 设置客户端最后登录的ip ipPolicyAccountService.ipValidate(user.getUserId(), user.getLastLoginIp()",
                Boolean.class,
                ()->ipPolicyAccountService.ipValidate(user.getUserId(), user.getLastLoginIp())
        );

        if(isNotAdmin && !aBoolean) {
            return CasStatus.STATUS_905;
        }

        // 账号有效期验证
        if(isNotAdmin && !accountTimelimitValidate(user)) {
            return CasStatus.STATUS_906;
        }

        // 登录成功: 记录最近登录时间和ip, 更新用户信息
        user.setLastLoginDate(new Date());
//        asyncTaskService.updateUserLastLoginInfo(user);
//        // 用户在线统计
//        asyncTaskService.updateOnlineUserCount(user);

        // 成功后不返回错误信息
        return null;
    }

    /**
     * 私有方法-密码验证
     * 和密码有效期验证
     * 和密码验证失败错误次数验证
     * 并更新用户的锁定状态
     */
    private CasStatus passwordValidate(String password, Boolean isAdmin, AccoutInfo user){
        // 其他用户(不是超管用户)
        Boolean isNotAdmin = !isAdmin;
        // 密码是否匹配
        Boolean isSuccess = PasswordCheckUtils.matches(password, user.getPassWord());

        // 密码有效期验证
        if(isNotAdmin && !passwordTimelimitValidate(user)){
            return CasStatus.STATUS_907;
        }

        if(!isSuccess){
            // 如果没设置阀值, 就不校验登录次数
            if(isNotAdmin && null != user.getLoginFailnum() && !user.getLoginFailnum().equals(0)) {
                Integer pwdErrCount = passwordErrorCountCache.get(user.getUserId());
                pwdErrCount = (null == pwdErrCount ? 0 : pwdErrCount) + 1;

                // 登录次数: 判断验证失败后, 如果失败次数达到阀值, 则清除登录失败次数缓存, 并锁定用户
                if (pwdErrCount.equals(user.getLoginFailnum())) {
                    passwordErrorCountCache.put(user.getUserId(), 0);
                    // 锁定用户
                    user.setUserStat("1");
                    // 更新用户信息
                    accoutInfoService.update(user);
                    // 抛出账号锁定异常信息
                    return CasStatus.STATUS_903;
                }
            }
            return CasStatus.STATUS_908;
        }

        return null;
    }

    /**
     * 账号有效期验证
     * 例如: 30天后过期, 创建时间至今日的间隔日期数量是否大于有效期限, 是则不允许登录
     */
    private Boolean accountTimelimitValidate(AccoutInfo user){
        if(user.getCreateTime() != null && user.getAccountTimelimit() != null){
            // 创建时间和密码更新时间至少的有一个有值的, 才能校验账号有效期
            Date createTime = user.getCreateTime();

            // 计算2个日期的间隔日期列表
            List<Date> betweenDates = DateUtils.getBetweenDates(
                    DateUtils.formatDate(createTime, "yyyy-MM-dd"),
                    DateUtils.formatDate(new Date(), "yyyy-MM-dd"));

            if(ListUtil.isNotEmpty(betweenDates) && betweenDates.size() > user.getAccountTimelimit().intValue()){
                return false;
            }
        }
        return true;
    }

    /**
     * 密码有效期验证
     * 根据密码策略最长时间判断当前密码是否过期
     * 如果没有密码策略 或 密码使用最长期限为空 或 密码最后修改时间 和 创建时间都未空, 则不验证
     */
    private Boolean passwordTimelimitValidate(AccoutInfo user){
        // 获取密码策略
        AccountPolicy accountPolicy = accountPolicyService.getOne();
        // 如果没有密码策略, 则不验证
        if(null == accountPolicy){
            return true;
        }

        // 密码使用最长期限, 如果为空, 则不验证
        Integer maxTime = accountPolicy.getMaxTime();
        if(null == maxTime){
            return true;
        }

        // 密码最后修改时间为空, 则取创建时间作为判断条件
        Date passUpdateTime = user.getPassUpdateTime();
        if(null == passUpdateTime){
            passUpdateTime = user.getCreateTime();
        }

        // 如果密码最后修改时间和创建时间都未空, 则不验证
        if(null == passUpdateTime){
            return true;
        }

        // 计算2个日期的间隔日期列表
        List<Date> betweenDates = DateUtils.getBetweenDates(
                DateUtils.formatDate(passUpdateTime, "yyyy-MM-dd"),
                DateUtils.formatDate(new Date(), "yyyy-MM-dd"));

        if(ListUtil.isNotEmpty(betweenDates) && betweenDates.size() > maxTime){
            return false;
        }

        return true;
    }

    /**
     * 私有方法-验证码验证
     * @param CaptchaSwitch 验证码开关， true开启验证码验证 false关闭验证码验证
     * @return
     */
    private CasStatus captchaValidate(boolean CaptchaSwitch, String captcha){
        // 验证码开关开启则验证验证码
        if(CaptchaSwitch) {
            Boolean isSuccess = captchaService.validateCacheCaptcha(captcha);
            if (null == isSuccess || !isSuccess) {
                return CasStatus.STATUS_901;
            }
        }
        return null;
    }



}
