package com.loit.v1.oauth.service;

import com.loit.common.constant.LoitHeaderConstant;
import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.CookieUtils;
import com.loit.loitcasclient.config.ClientAppConfig;
import com.loit.loitcasclient.utils.ClientUserUtil;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.OpenAccessTokenDTO;
import com.loit.v1.api.util.DateTotalUtils.DateTotalUtils;
import com.loit.v1.oauth.data.CasStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletResponse;

/**
 * rest 方式登录服务
 */
@Service
public class RestLoginOauthService {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private LoginOauthValidateService loginOauthValidateService;
    @Autowired
    protected ClientAppConfig clientAppConfig;
    @Autowired
    protected RestTemplate restTemplate;
    @Autowired
    private ClientUserUtil userUtil;

    /**
     * Rest接口级登录验证方法
     * 验证账号
     * <p>
     * 101 失败
     * 901 验证码错误
     * 902 账号不存在
     * 903 账号已被锁定
     * 904 账号被管理员禁止
     * 905 登录IP受限
     * 906 账号已过期
     * 907 密码已过期
     * 908 密码错误
     * 909 错误9
     * 910 错误10
     *
     * @param userName 登录用户名
     * @return
     */
    public CasStatus login(String userName) {
        try {
            // 登录验证
            return loginOauthValidateService.loginValidate(userName);
        } catch (Exception e) {
            logger.error("用户认证错误", e);
            return CasStatus.STATUS_101;
        }
    }

    /**
     * Rest接口级登录验证方法
     * 验证账号和密码
     * <p>
     * 101 失败
     * 901 验证码错误
     * 902 账号不存在
     * 903 账号已被锁定
     * 904 账号被管理员禁止
     * 905 登录IP受限
     * 906 账号已过期
     * 907 密码已过期
     * 908 密码错误
     * 909 错误9
     * 910 错误10
     *
     * @param userName 登录用户名
     * @param password 登录密码
     * @return
     */
    public CasStatus login(String userName, String password) {
        try {
            // 登录验证
            return loginOauthValidateService.loginValidate(userName, password);
        } catch (Exception e) {
            logger.error("用户认证错误", e);
            return CasStatus.STATUS_101;
        }
    }

    /**
     * rest登录验证成功后的令牌处理程序
     *
     * @param userId        登录用户名
     * @param password      用户密码
     * @param loginType     有密登录为 1，无密登录为 0
     * @param loginCastatus 登录验证的状态
     * @param response      响应类用来存入cookie
     * @return 返回登录用户对象信息
     * <p>
     * 注：CAS Oauth2 密码模式，这种在头信息中传入service或X-service值来代替应用程序的URl。头信息中必须与客户端ID的注册表中的OAuth Serice匹配
     */
    public AjaxJson restAccessTokenHandler(String userId, String password, CasStatus loginCastatus, String loginType, HttpServletResponse response) {
        try {
            long one = System.currentTimeMillis(); //获取开始时间

            AjaxJson json = new AjaxJson();

            if (null != loginCastatus) {
                return AjaxJson.returnExceptionInfo(loginCastatus.getValue(), loginCastatus.getMsg());
            }
            long two = System.currentTimeMillis();
            System.out.println("restAccessTokenHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$1111: " + (two - one));
            // 令牌获取和处理
            OpenAccessTokenDTO openAccessTokenDTO = this.restAccessTokenNoUserHandler(userId, password, loginType, response);
            long three = System.currentTimeMillis();
            System.out.println("restAccessTokenHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$2222: " + (three - two));

            // accessToken存入cookie和header中
            logger.info("response set cookie " + LoitHeaderConstant.TOKEN_NAME + "：" + openAccessTokenDTO.getAccess_token());
            CookieUtils.setCookie(response, LoitHeaderConstant.TOKEN_NAME, openAccessTokenDTO.getAccess_token());
            response.setHeader(LoitHeaderConstant.TOKEN_NAME, openAccessTokenDTO.getAccess_token());
            long four = System.currentTimeMillis();
            System.out.println("restAccessTokenHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$333: " + (four - three));
            // 用令牌缓存userId
//            logger.info("cache set accessToken："+openAccessTokenDTO.getAccess_token()+" to userId：" + userId + " timeOut：" + openAccessTokenDTO.getExpires_in());
            //CacheUtils.set(CacheConstants.ACCESS_TOKEN_TO_USERID.replace("{accessToken}", openAccessTokenDTO.getAccess_token()), userId, Long.parseLong(openAccessTokenDTO.getExpires_in()));

            CacheUtils.set(openAccessTokenDTO.getAccess_token(), userId, Long.parseLong(openAccessTokenDTO.getExpires_in()));

            //TODO delete
            //String userIds = userUtil.getCasServerUserId(openAccessTokenDTO.getAccess_token());

            long five = System.currentTimeMillis();
            System.out.println("restAccessTokenHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$444: " + (five - four));
            //logger.info("获取到用户Id" + userIds);
            //返回登录用户对象信息
            User user = DateTotalUtils.printR(
                    "2.2.返回登录用户对象信息 UserUtils.get(userId)",
                    User.class,
                    () -> UserUtils.get(userId)
            );
            json.put("user", user);
            long six = System.currentTimeMillis();
            System.out.println("restAccessTokenHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$555: " + (six - five));

            return json;
        } catch (RestClientException e) {
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
        }
    }

    /**
     * 令牌获取和处理
     *
     * @param userId    登录用户名
     * @param password  用户密码
     * @param loginType 有密登录为 1，无密登录为 0
     * @param response  响应类用来存入cookie
     * @return 返回令牌对象
     */
    public OpenAccessTokenDTO restAccessTokenNoUserHandler(String userId, String password, String loginType, HttpServletResponse response) {

        long one = System.currentTimeMillis(); //获取开始时间
        // cas server rest登录接口url
        String url = "";
        if (loginType.equals("0")) {
            url = clientAppConfig.accessTokenForNoPasswordUrl().replace("{username}", userId);
        } else {
            url = clientAppConfig.accessTokenForPasswordUrl().replace("{username}", userId).replace("{password}", password);
        }

        long two = System.currentTimeMillis();
        System.out.println("restAccessTokenNoUserHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$1111: " + (two - one));


        // 必须参数, 否则登录接口不能通过, service内容与cas server鉴权文件内容保持一致
        HttpHeaders headers = new HttpHeaders();
        headers.add("service", "http://localhost?userId=" + userId);
        long three = System.currentTimeMillis();
        System.out.println("restAccessTokenNoUserHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$222: " + (three - two));

        // 获取令牌json
        HttpEntity<String> requestEntity = new HttpEntity<>(null, headers);
        String finalUrl = url;
        ResponseEntity<OpenAccessTokenDTO> accessTokenEntity = DateTotalUtils.printR(
                "2.1.cas server rest登录获取令牌 restTemplate.exchange(url, HttpMethod.GET, requestEntity, OpenAccessTokenDTO.class)",
                ResponseEntity.class,
                () -> restTemplate.exchange(finalUrl, HttpMethod.GET, requestEntity, OpenAccessTokenDTO.class)
        );

        long four = System.currentTimeMillis();
        System.out.println("restAccessTokenNoUserHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$333: " + (four - three));

        OpenAccessTokenDTO openAccessTokenDTO = accessTokenEntity.getBody();

        // accessToken存入cookie和header中
        logger.info("response set cookie " + LoitHeaderConstant.TOKEN_NAME + "：" + openAccessTokenDTO.getAccess_token());
        CookieUtils.setCookie(response, LoitHeaderConstant.TOKEN_NAME, openAccessTokenDTO.getAccess_token());
        response.setHeader(LoitHeaderConstant.TOKEN_NAME, openAccessTokenDTO.getAccess_token());

        long five = System.currentTimeMillis();
        System.out.println("restAccessTokenNoUserHandler$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$444: " + (five - four));

        return openAccessTokenDTO;
    }
}
