package com.loit.v1.oauth.service;

import com.loit.v1.modules.config.SwitchConfig;
import com.loit.v1.oauth.data.CasStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.Base64Utils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestHeader;

import java.io.UnsupportedEncodingException;

/**
 * 页面登录服务
 */
@Service
public class WebLoginOauthService {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private LoginOauthValidateService loginOauthValidateService;
    @Autowired
    private RestLoginOauthService restLoginOauthService;
    @Autowired
    private SwitchConfig switchConfig;

    /**
     * 免登录验证
     * cas server 需要配置该登录验证接口调用，非人为调用
     */
    public Object noPasswordLogin(@RequestHeader HttpHeaders httpHeaders) {
        logger.info("开始验证服务");

        try {
            // 获取CAS服务器传来的用户名密码
            UserTemp userTemp = getCasServerLoginInfoFormHeader(httpHeaders);
            CasStatus exceptionCasStatus = restLoginOauthService.login(userTemp.username);

            if (null != exceptionCasStatus) {
                return new ResponseEntity(exceptionCasStatus.getValue(), HttpStatus.UNAUTHORIZED);
            }

            // 成功返回json
            return "{\"@class\":\"org.apereo.cas.authentication.principal.SimplePrincipal\",\"id\":\"" + userTemp.username + "\",\"attributes\":{}}";
        } catch (UnsupportedEncodingException e) {
            logger.error("用户认证错误", e);
            return new ResponseEntity(HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * 页面级登录验证方法-账号和密码和验证码校验
     * cas server 需要配置该登录验证接口调用，非人为调用
     * <p>
     * 901 验证码错误
     * 902 账号不存在
     * 903 账号已被锁定
     * 904 账号被管理员禁止
     * 905 登录IP受限
     * 906 账号已过期
     * 907 密码已过期
     * 908 密码错误
     * 909 错误9
     * 910 错误10
     */
    public Object login(@RequestHeader HttpHeaders httpHeaders) {
        logger.info("开始验证服务");
        long one = System.currentTimeMillis(); //获取开始时间
        try {
            // 获取CAS服务器传来的用户名密码
            UserTemp userTemp = getCasServerLoginInfoFormHeader(httpHeaders);
            String returnStr = "{\"@class\":\"org.apereo.cas.authentication.principal.SimplePrincipal\",\"id\":\"" + userTemp.username + "\",\"attributes\":{}}";
            String loginTypeHeader = httpHeaders.getFirst("loginType");
            //TODO  isAppLogin
            //APP登录已经在 restLoginOauthService.login(userId, password)中登录过, 直接返回。
            if (!StringUtils.isEmpty(loginTypeHeader)) {
                String loginType = new String(Base64Utils.decodeFromString(loginTypeHeader), "UTF-8");
                if (isAppLogin(loginType)){

                    long two = System.currentTimeMillis();
                    System.out.println("&&&&&&&&&&&&&&&%%%%%%%%%%%&&&&&&&&&&&&&&&&&&&&&&&&&&login$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$: " + (two - one));
                    return returnStr;
                }
            }


            String nopasswordHeader = httpHeaders.getFirst("nopassword");
            boolean captchaSwitch = switchConfig.getCaptchaSwitch();

            // 登录验证, 失败返回错误信息
            CasStatus exceptionCasStatus = null;
            if (!StringUtils.isEmpty(nopasswordHeader)) {
                String nopassword = new String(Base64Utils.decodeFromString(nopasswordHeader), "UTF-8");
                //无密登录
                if (nopassword.equals("1")) {
                    exceptionCasStatus = restLoginOauthService.login(userTemp.username);
                } else {
                    exceptionCasStatus = loginOauthValidateService.loginValidate(userTemp.username, userTemp.password, captchaSwitch, userTemp.captcha);

                }

            } else {
                exceptionCasStatus = loginOauthValidateService.loginValidate(userTemp.username, userTemp.password, captchaSwitch, userTemp.captcha);

            }

            if (null != exceptionCasStatus) {
                return new ResponseEntity(exceptionCasStatus.getValue(), HttpStatus.UNAUTHORIZED);
            }


            // 成功返回json
            return returnStr;

        } catch (Exception e) {
            logger.error("用户认证错误", e);
            return new ResponseEntity(HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * loginType 为0 表示app登录
     */
    private boolean isAppLogin(String loginType) {
        return "0".equals(loginType);
    }

    /**
     * 私有方法-接收cas server的header传来的用户名和密码
     * 这允许CAS服务器通过POST到达远程REST端点以验证凭据.
     * 凭证通过Authorization标头传递，该标头的值为Basic XYZ，其中XYZ是凭证的Base64编码版本.
     *
     * @param httpHeaders
     * @return
     * @throws UnsupportedEncodingException
     */
    private UserTemp getCasServerLoginInfoFormHeader(HttpHeaders httpHeaders) throws UnsupportedEncodingException {
        //cas服务端会通过把用户信息放在请求头authorization中，并且通过Basic认证方式加密
        String authorization = httpHeaders.getFirst("authorization");
        if (StringUtils.isEmpty(authorization)) {
            return null;
        }

        String baseCredentials = authorization.split(" ")[1];

        //用户名:验证码:密码
        String usernamePassword = new String(Base64Utils.decodeFromString(baseCredentials), "UTF-8");
        String[] credentials = usernamePassword.split(":");

        return new UserTemp(credentials[0], credentials[1], credentials[2]);
    }

    /**
     * 内部类-从请求头中获取用户名和密码
     */
    class UserTemp {
        public String username;
        public String password;
        public String captcha;

        public UserTemp(String username, String captcha, String password) {
            this.username = username;
            this.password = password;
            this.captcha = captcha;
        }
    }
}
